@@ -91,6 +91,7 @@ static int gzseek(gzFile file, long offset, int whence) {
  *          dist: float
  *          rot: quaternion
  *          ofs: offset
+ *          ortho: bool
  *
  */
 
@@ -224,7 +225,7 @@ static void chunk_write_all(gzFile out, const char *type,
     write_int32(out, 0);        // CRC XXX: todo.
 }
 
-void save_to_file(goxel_t *goxel, const char *path, bool with_preview)
+void save_to_file(const char *path, bool with_preview)
 {
     // XXX: remove all empty blocks before saving.
     LOG_I("Save to %s", path);
@@ -340,6 +341,8 @@ void save_to_file(goxel_t *goxel, const char *path, bool with_preview)
                                sizeof(camera->rot));
         chunk_write_dict_value(&c, out, "ofs", &camera->ofs,
                                sizeof(camera->ofs));
+        chunk_write_dict_value(&c, out, "ortho", &camera->ortho,
+                               sizeof(camera->ortho));
         if (camera == goxel->image->active_camera)
             chunk_write_dict_value(&c, out, "active", NULL, 0);
 
@@ -400,12 +403,12 @@ static block_hash_t *hash_find_at(block_hash_t *hash, int index)
     return hash;
 }
 
-int load_from_file(goxel_t *goxel, const char *path)
+int load_from_file(const char *path)
 {
     layer_t *layer, *layer_tmp;
     block_hash_t *blocks_table = NULL, *data, *data_tmp;
     gzFile in;
-    char magic[4];
+    char magic[4] = {};
     uint8_t *voxel_data;
     int nb_blocks;
     int w, h, bpp;
@@ -418,16 +421,15 @@ int load_from_file(goxel_t *goxel, const char *path)
     uint64_t uid = 1;
     camera_t *camera;
 
-    LOG_I("Load from file %s", path);
     in = gzopen(path, "rb");
     if (!in) return -1;
 
     gzread(in, magic, 4);
-    assert(strncmp(magic, "GOX ", 4) == 0);
+    if (strncmp(magic, "GOX ", 4) != 0) goto error;
     version = read_int32(in);
     if (version > VERSION) {
         LOG_W("Cannot open gox file version %d", version);
-        return -1;
+        goto error;
     }
 
     // Remove all layers.
@@ -437,6 +439,7 @@ int load_from_file(goxel_t *goxel, const char *path)
         free(layer);
     }
     goxel->image->layers = NULL;
+    memset(&goxel->image->box, 0, sizeof(goxel->image->box));
 
     while (chunk_read_start(&c, in)) {
         if (strncmp(c.type, "BL16", 4) == 0) {
@@ -503,6 +506,8 @@ int load_from_file(goxel_t *goxel, const char *path)
                     memcpy(&camera->rot, dict_value, dict_value_size);
                 if (strcmp(dict_key, "ofs") == 0)
                     memcpy(&camera->ofs, dict_value, dict_value_size);
+                if (strcmp(dict_key, "ortho") == 0)
+                    memcpy(&camera->ortho, dict_value, dict_value_size);
                 if (strcmp(dict_key, "active") == 0) {
                     goxel->image->active_camera = camera;
                     camera_set(&goxel->camera, camera);
@@ -531,9 +536,20 @@ int load_from_file(goxel_t *goxel, const char *path)
     }
 
     goxel->image->path = strdup(path);
-    goxel_update_meshes(goxel, -1);
+    goxel_update_meshes(-1);
     gzclose(in);
+
+    // Update plane, snap mask and camera pos not to confuse people.
+    plane_from_vectors(goxel->plane, goxel->image->box[3],
+                       VEC(1, 0, 0), VEC(0, 1, 0));
+    if (box_is_null(goxel->image->box)) goxel->snap_mask |= SNAP_PLANE;
+    camera_fit_box(&goxel->camera, goxel->image->box);
+
     return 0;
+
+error:
+    gzclose(in);
+    return -1;
 }
 
 static void action_open(const char *path)
@@ -545,14 +561,14 @@ static void action_open(const char *path)
     if (!path) return;
     image_delete(goxel->image);
     goxel->image = image_new();
-    load_from_file(goxel, path);
+    load_from_file(path);
 }
 
 ACTION_REGISTER(open,
     .help = "Open an image",
     .cfunc = action_open,
     .csig = "vp",
-    .shortcut = "Ctrl O",
+    .default_shortcut = "Ctrl O",
 )
 
 static void save_as(const char *path, bool with_preview)
@@ -566,7 +582,7 @@ static void save_as(const char *path, bool with_preview)
         free(goxel->image->path);
         goxel->image->path = strdup(path);
     }
-    save_to_file(goxel, goxel->image->path, with_preview);
+    save_to_file(goxel->image->path, with_preview);
 }
 
 ACTION_REGISTER(save_as,
@@ -584,5 +600,5 @@ ACTION_REGISTER(save,
     .help = "Save the image",
     .cfunc = save,
     .csig = "vpi",
-    .shortcut = "Ctrl S"
+    .default_shortcut = "Ctrl S"
 )
