@@ -122,7 +122,7 @@ static int vxl_import(const char *path)
 
     mesh_blit(goxel->image->active_layer->mesh, (uint8_t*)cube,
               -w / 2, -h / 2, -d / 2, w, h, d, NULL);
-    goxel_update_meshes(goxel, -1);
+    goxel_update_meshes(-1);
     if (box_is_null(goxel->image->box)) {
         bbox_from_extents(goxel->image->box, vec3_zero, w / 2, h / 2, d / 2);
     }
