@@ -58,9 +58,9 @@ static bool unproject_delta(const float win[3], const float model[4][4],
 }
 
 // XXX: lot of cleanup to do here.
-bool goxel_unproject_on_plane(goxel_t *goxel, const float viewport[4],
-                              const float pos[2], const float plane[4][4],
-                              float out[3], float normal[3])
+static bool goxel_unproject_on_plane(
+        const float viewport[4], const float pos[2], const float plane[4][4],
+        float out[3], float normal[3])
 {
     // If the angle between the screen and the plane is close to 90 deg,
     // the projection fails.  This prevents projecting too far away.
@@ -80,9 +80,10 @@ bool goxel_unproject_on_plane(goxel_t *goxel, const float viewport[4],
     return true;
 }
 
-bool goxel_unproject_on_box(goxel_t *goxel, const float viewport[4],
-                     const float pos[2], const float box[4][4], bool inside,
-                     float out[3], float normal[3], int *face)
+static bool goxel_unproject_on_box(
+        const float viewport[4], const float pos[2],
+        const float box[4][4], bool inside, float out[3],
+        float normal[3], int *face)
 {
     int f;
     float wpos[3] = {pos[0], pos[1], 0};
@@ -112,9 +113,9 @@ bool goxel_unproject_on_box(goxel_t *goxel, const float viewport[4],
     return false;
 }
 
-bool goxel_unproject_on_mesh(goxel_t *goxel, const float view[4],
-                             const float pos[2], mesh_t *mesh,
-                             float out[3], float normal[3])
+static bool goxel_unproject_on_mesh(
+        const float view[4], const float pos[2], mesh_t *mesh,
+        float out[3], float normal[3])
 {
     float view_size[2] = {view[2], view[3]};
     // XXX: No need to render the fbo if it is not dirty.
@@ -167,7 +168,7 @@ bool goxel_unproject_on_mesh(goxel_t *goxel, const float view[4],
 }
 
 
-int goxel_unproject(goxel_t *goxel, const float viewport[4],
+int goxel_unproject(const float viewport[4],
                     const float pos[2], int snap_mask, float offset,
                     float out[3], float normal[3])
 {
@@ -178,7 +179,7 @@ int goxel_unproject(goxel_t *goxel, const float viewport[4],
 
     // If tool_plane is set, we specifically use it.
     if (!plane_is_null(goxel->tool_plane)) {
-        r = goxel_unproject_on_plane(goxel, viewport, pos,
+        r = goxel_unproject_on_plane(viewport, pos,
                                      goxel->tool_plane, out, normal);
         ret = r ? SNAP_PLANE : 0;
         goto end;
@@ -187,31 +188,31 @@ int goxel_unproject(goxel_t *goxel, const float viewport[4],
     for (i = 0; i < 7; i++) {
         if (!(snap_mask & (1 << i))) continue;
         if ((1 << i) == SNAP_MESH) {
-            r = goxel_unproject_on_mesh(goxel, viewport, pos,
+            r = goxel_unproject_on_mesh(viewport, pos,
                                         goxel->layers_mesh, p, n);
         }
         if ((1 << i) == SNAP_PLANE)
-            r = goxel_unproject_on_plane(goxel, viewport, pos,
+            r = goxel_unproject_on_plane(viewport, pos,
                                          goxel->plane, p, n);
         if ((1 << i) == SNAP_SELECTION_IN)
-            r = goxel_unproject_on_box(goxel, viewport, pos,
+            r = goxel_unproject_on_box(viewport, pos,
                                        goxel->selection, true,
                                        p, n, NULL);
         if ((1 << i) == SNAP_SELECTION_OUT)
-            r = goxel_unproject_on_box(goxel, viewport, pos,
+            r = goxel_unproject_on_box(viewport, pos,
                                        goxel->selection, false,
                                        p, n, NULL);
         if ((1 << i) == SNAP_LAYER_OUT) {
             mesh_get_box(goxel->image->active_layer->mesh, true, box);
-            r = goxel_unproject_on_box(goxel, viewport, pos, box, false,
+            r = goxel_unproject_on_box(viewport, pos, box, false,
                                        p, n, NULL);
         }
         if ((1 << i) == SNAP_IMAGE_BOX)
-            r = goxel_unproject_on_box(goxel, viewport, pos,
+            r = goxel_unproject_on_box(viewport, pos,
                                        goxel->image->box, true,
                                        p, n, NULL);
         if ((1 << i) == SNAP_IMAGE_BOX)
-            r = goxel_unproject_on_box(goxel, viewport, pos,
+            r = goxel_unproject_on_box(viewport, pos,
                                        goxel->image->box, true,
                                        p, n, NULL);
         if ((1 << i) == SNAP_CAMERA) {
@@ -253,29 +254,18 @@ static int on_pan(const gesture_t *gest, void *user);
 static int on_rotate(const gesture_t *gest, void *user);
 static int on_hover(const gesture_t *gest, void *user);
 
-void goxel_init(goxel_t *gox)
+void goxel_init(void)
 {
-    goxel = gox;
-    memset(goxel, 0, sizeof(*goxel));
+    goxel = calloc(1, sizeof(*goxel));
 
     render_init();
     shapes_init();
     sound_init();
-    quat_set_identity(goxel->camera.rot);
-    goxel->camera.dist = 128;
-    goxel->camera.aspect = 1;
-    quat_irotate(goxel->camera.rot, -M_PI / 4, 1, 0, 0);
-    quat_irotate(goxel->camera.rot, -M_PI / 4, 0, 0, 1);
-
-    goxel->image = image_new();
+    cycles_init();
+    model3d_init();
 
     goxel->layers_mesh = mesh_new();
     goxel->render_mesh = mesh_new();
-    goxel_update_meshes(goxel, -1);
-
-    vec4_set(goxel->back_color, 70, 70, 70, 255);
-    vec4_set(goxel->grid_color, 19, 19, 19, 255);
-    vec4_set(goxel->image_box_color, 204, 204, 255, 255);
 
     // Load and set default palette.
     palette_load_all(&goxel->palettes);
@@ -285,28 +275,6 @@ void goxel_init(goxel_t *gox)
     }
     goxel->palette = goxel->palette ?: goxel->palettes;
 
-    action_exec2("tool_set_brush", "");
-    goxel->tool_radius = 0.5;
-    goxel->painter = (painter_t) {
-        .shape = &shape_cube,
-        .mode = MODE_OVER,
-        .smoothness = 0,
-        .color = {255, 255, 255, 255},
-    };
-    goxel->rend = (renderer_t) {
-        .light = {
-            .pitch = 10 * DD2R,
-            .yaw = 120 * DD2R,
-            .fixed = true,
-            .intensity = 1.
-        },
-    };
-    render_get_default_settings(0, NULL, &goxel->rend.settings);
-
-    model3d_init();
-    plane_from_vectors(goxel->plane, vec3_zero, VEC(1, 0, 0), VEC(0, 1, 0));
-    goxel->snap_mask = SNAP_PLANE | SNAP_MESH | SNAP_IMAGE_BOX;
-
     goxel->gestures.drag = (gesture_t) {
         .type = GESTURE_DRAG,
         .button = 0,
@@ -327,22 +295,79 @@ void goxel_init(goxel_t *gox)
         .callback = on_hover,
     };
 
-    action_exec2("settings_load", "");
+    goxel_reset();
 }
 
-void goxel_release(goxel_t *goxel)
+void goxel_reset(void)
 {
+    image_delete(goxel->image);
+    goxel->image = image_new();
+    quat_set_identity(goxel->camera.rot);
+    goxel->camera.dist = 128;
+    goxel->camera.aspect = 1;
+    quat_irotate(goxel->camera.rot, -M_PI / 4, 1, 0, 0);
+    quat_irotate(goxel->camera.rot, -M_PI / 4, 0, 0, 1);
+    action_exec2("settings_load", "");
+
+    camera_fit_box(&goxel->camera, goxel->image->box);
+    // Put plane horizontal at the center of the image box.
+    plane_from_vectors(goxel->plane, goxel->image->box[3],
+                       VEC(1, 0, 0), VEC(0, 1, 0));
+
+    vec4_set(goxel->back_color, 70, 70, 70, 255);
+    vec4_set(goxel->grid_color, 19, 19, 19, 255);
+    vec4_set(goxel->image_box_color, 204, 204, 255, 255);
+
+    goxel_update_meshes(-1);
+
+    action_exec2("tool_set_brush", "");
+    goxel->tool_radius = 0.5;
+    goxel->painter = (painter_t) {
+        .shape = &shape_cube,
+        .mode = MODE_OVER,
+        .smoothness = 0,
+        .color = {255, 255, 255, 255},
+    };
+
+    // Set symmetry origin to the center of the image.
+    mat4_mul_vec3(goxel->image->box, VEC(0, 0, 0),
+                  goxel->painter.symmetry_origin);
+
+    goxel->rend = (renderer_t) {
+        .light = {
+            .pitch = 10 * DD2R,
+            .yaw = 120 * DD2R,
+            .fixed = true,
+            .intensity = 1.
+        },
+    };
+    render_get_default_settings(0, NULL, &goxel->rend.settings);
+
+    goxel->snap_mask = SNAP_MESH | SNAP_IMAGE_BOX;
+}
+
+void goxel_release(void)
+{
+    cycles_release();
     proc_release(&goxel->proc);
     gui_release();
 }
 
-void goxel_iter(goxel_t *goxel, inputs_t *inputs)
+static void update_window_title(void)
+{
+    char buf[1024];
+    sprintf(buf, "Goxel %s%s %s", GOXEL_VERSION_STR, DEBUG ? " (debug)" : "",
+            goxel->image->path ?: "");
+    sys_set_window_title(buf);
+}
+
+int goxel_iter(inputs_t *inputs)
 {
     double time = sys_get_time();
     goxel->fps = mix(goxel->fps, 1.0 / (time - goxel->frame_time), 0.1);
     goxel->frame_time = time;
-    goxel_set_help_text(goxel, NULL);
-    goxel_set_hint_text(goxel, NULL);
+    goxel_set_help_text(NULL);
+    goxel_set_hint_text(NULL);
     goxel->screen_size[0] = inputs->window_size[0];
     goxel->screen_size[1] = inputs->window_size[1];
     goxel->screen_scale = inputs->scale;
@@ -353,9 +378,12 @@ void goxel_iter(goxel_t *goxel, inputs_t *inputs)
         camera_set(goxel->image->active_camera, &goxel->camera);
     mat4_copy(goxel->camera.view_mat, goxel->rend.view_mat);
     mat4_copy(goxel->camera.proj_mat, goxel->rend.proj_mat);
-    gui_iter(goxel, inputs);
+    gui_iter(inputs);
     sound_iter();
+    update_window_title();
+
     goxel->frame_count++;
+    return goxel->quit ? 1 : 0;
 }
 
 static void compute_view_rotation(const float rot[4],
@@ -381,15 +409,16 @@ static void set_cursor_hint(cursor_t *curs)
 {
     const char *snap_str = NULL;
     if (!curs->snaped) {
-        goxel_set_hint_text(goxel, NULL);
+        goxel_set_hint_text(NULL);
         return;
     }
     if (curs->snaped == SNAP_MESH) snap_str = "mesh";
     if (curs->snaped == SNAP_PLANE) snap_str = "plane";
-    if (IS_IN(curs->snaped, SNAP_SELECTION_IN, SNAP_SELECTION_OUT))
-        snap_str = "selection";
+    if (curs->snaped == SNAP_IMAGE_BOX) snap_str = "bounding box";
+    if (    curs->snaped == SNAP_SELECTION_IN ||
+            curs->snaped == SNAP_SELECTION_OUT) snap_str = "selection";
 
-    goxel_set_hint_text(goxel, "[%.0f %.0f %.0f] (%s)",
+    goxel_set_hint_text("[%.0f %.0f %.0f] (%s)",
             curs->pos[0] - 0.5, curs->pos[1] - 0.5, curs->pos[2] - 0.5,
             snap_str);
 }
@@ -403,7 +432,7 @@ static int on_drag(const gesture_t *gest, void *user)
         c->flags &= ~CURSOR_PRESSED;
 
     c->snaped = goxel_unproject(
-            goxel, gest->viewport, gest->pos, c->snap_mask,
+            gest->viewport, gest->pos, c->snap_mask,
             c->snap_offset, c->pos, c->normal);
 
     // Set some default values.  The tools can override them.
@@ -417,7 +446,7 @@ static int on_drag(const gesture_t *gest, void *user)
 static int on_pan(const gesture_t *gest, void *user)
 {
     if (gest->state == GESTURE_BEGIN) {
-        quat_copy(goxel->camera.ofs, goxel->move_origin.camera_ofs);
+        vec3_copy(goxel->camera.ofs, goxel->move_origin.camera_ofs);
         vec2_copy(gest->pos, goxel->move_origin.pos);
     }
     float wpos[3] = {gest->pos[0], gest->pos[1], 0};
@@ -454,9 +483,8 @@ static int on_rotate(const gesture_t *gest, void *user)
 static int on_hover(const gesture_t *gest, void *user)
 {
     cursor_t *c = &goxel->cursor;
-    c->snaped = goxel_unproject(
-                    goxel, gest->viewport, gest->pos, c->snap_mask,
-                    c->snap_offset, c->pos, c->normal);
+    c->snaped = goxel_unproject(gest->viewport, gest->pos, c->snap_mask,
+                                c->snap_offset, c->pos, c->normal);
     set_cursor_hint(c);
     c->flags &= ~CURSOR_PRESSED;
     // Set some default values.  The tools can override them.
@@ -464,13 +492,13 @@ static int on_hover(const gesture_t *gest, void *user)
     c->snap_mask = goxel->snap_mask;
     set_flag(&c->snap_mask, SNAP_ROUNDED, goxel->painter.smoothness == 0);
     c->snap_offset = 0;
+    set_flag(&c->flags, CURSOR_OUT, gest->state == GESTURE_END);
     return 0;
 }
 
 
 // XXX: Cleanup this.
-void goxel_mouse_in_view(goxel_t *goxel, const float viewport[4],
-                         const inputs_t *inputs)
+void goxel_mouse_in_view(const float viewport[4], const inputs_t *inputs)
 {
     float x_axis[4], q[4], p[3], n[3];
     gesture_t *gests[] = {&goxel->gestures.drag,
@@ -487,13 +515,14 @@ void goxel_mouse_in_view(goxel_t *goxel, const float viewport[4],
                          !box_is_null(goxel->image->box) ?
                          &goxel->image->box : NULL;
 
-    tool_iter(goxel->tool, viewport);
+    if (!goxel->no_edit)
+        tool_iter(goxel->tool, viewport);
 
     if (inputs->mouse_wheel) {
         goxel->camera.dist /= pow(1.1, inputs->mouse_wheel);
 
         // Auto adjust the camera rotation position.
-        if (goxel_unproject_on_mesh(goxel, viewport, inputs->touches[0].pos,
+        if (goxel_unproject_on_mesh(viewport, inputs->touches[0].pos,
                                     goxel->layers_mesh, p, n)) {
             camera_set_target(&goxel->camera, p);
         }
@@ -519,14 +548,14 @@ void goxel_mouse_in_view(goxel_t *goxel, const float viewport[4],
     }
     // C: recenter the view:
     if (inputs->keys['C']) {
-        if (goxel_unproject_on_mesh(goxel, viewport, inputs->touches[0].pos,
+        if (goxel_unproject_on_mesh(viewport, inputs->touches[0].pos,
                                     goxel->layers_mesh, p, n)) {
             camera_set_target(&goxel->camera, p);
         }
     }
 }
 
-void goxel_render(goxel_t *goxel)
+void goxel_render(void)
 {
     GL(glViewport(0, 0, goxel->screen_size[0] * goxel->screen_scale,
                         goxel->screen_size[1] * goxel->screen_scale));
@@ -538,7 +567,7 @@ void goxel_render(goxel_t *goxel)
     gui_render();
 }
 
-static void render_export_viewport(goxel_t *goxel, const float viewport[4])
+static void render_export_viewport(const float viewport[4])
 {
     // Render the export viewport.
     int w = goxel->image->export_width;
@@ -554,7 +583,76 @@ static void render_export_viewport(goxel_t *goxel, const float viewport[4])
     render_rect(&goxel->rend, plane, EFFECT_STRIP);
 }
 
-void goxel_render_view(goxel_t *goxel, const float viewport[4])
+static void render_view_cycles(const float viewport[4])
+{
+    int rect[4] = {(int)viewport[0],
+                   (int)(goxel->screen_size[1] - viewport[1] - viewport[3]),
+                   (int)viewport[2],
+                   (int)viewport[3]};
+    int w = viewport[2];
+    int h = viewport[3];
+    uint8_t *buf = calloc(4, w * h);
+    cycles_render(buf, &w, &h, &goxel->camera, NULL, false);
+    render_img2(&goxel->rend, buf, w, h, 4, NULL,
+                EFFECT_NO_SHADING | EFFECT_PROJ_SCREEN);
+    free(buf);
+    render_submit(&goxel->rend, rect, goxel->back_color);
+}
+
+void goxel_render_export_view(const float viewport[4])
+{
+    typeof(goxel->render_task) *task = &goxel->render_task;
+    int i, w, h, bpp = 4;
+    uint8_t *tmp;
+    float a, mat[4][4];
+    int rect[4] = {(int)viewport[0],
+                   (int)(goxel->screen_size[1] - viewport[1] - viewport[3]),
+                   (int)viewport[2],
+                   (int)viewport[3]};
+    camera_t camera = goxel->camera;
+
+    // Recreate the buffer if needed.
+    if (    !task->buf ||
+            task->w != goxel->image->export_width ||
+            task->h != goxel->image->export_height) {
+        free(task->buf);
+        task->w = goxel->image->export_width;
+        task->h = goxel->image->export_height;
+        task->buf = calloc(task->w * task->h, 4);
+    }
+
+    if (!task->status) task->status = 1;
+
+    w = task->w;
+    h = task->h;
+    camera.aspect = (float)w / h;
+    camera_update(&camera);
+    cycles_render(task->buf, &w, &h, &camera, &task->progress,
+                  task->force_restart);
+    task->force_restart = false;
+    mat4_set_identity(mat);
+    a = 1.0 * w / h / rect[2] * rect[3];
+    mat4_iscale(mat, min(a, 1.f), min(1.f / a, 1.f), 1);
+    render_img2(&goxel->rend, task->buf, w, h, 4, mat,
+                EFFECT_NO_SHADING | EFFECT_PROJ_SCREEN | EFFECT_ANTIALIASING);
+    render_submit(&goxel->rend, rect, goxel->back_color);
+    if (task->progress == 1) { // Finished.
+        if (*task->output) {
+            tmp = calloc(w * h, bpp);
+            // Flip output y.
+            for (i = 0; i < h; i++) {
+                memcpy(&tmp[i * (size_t)w * bpp],
+                    &task->buf[((size_t)h - i - 1) * (size_t)w * bpp],
+                    bpp * (size_t)w);
+            }
+            img_write(tmp, w, h, bpp, task->output);
+            free(tmp);
+        }
+        task->status = 2;
+    }
+}
+
+void goxel_render_view(const float viewport[4])
 {
     layer_t *layer;
     renderer_t *rend = &goxel->rend;
@@ -563,6 +661,11 @@ void goxel_render_view(goxel_t *goxel, const float viewport[4])
     goxel->camera.aspect = viewport[2] / viewport[3];
     camera_update(&goxel->camera);
 
+    if (goxel->use_cycles) {
+        render_view_cycles(viewport);
+        return;
+    }
+
     render_mesh(rend, goxel->render_mesh, 0);
     if (!box_is_null(goxel->image->active_layer->box))
         render_box(rend, goxel->image->active_layer->box,
@@ -592,17 +695,24 @@ void goxel_render_view(goxel_t *goxel, const float viewport[4])
         mesh_get_box(goxel->layers_mesh, false, b);
         render_box(rend, b, c, EFFECT_WIREFRAME);
     }
-    if (!goxel->plane_hidden)
-        render_plane(rend, goxel->plane, goxel->grid_color);
+    if (goxel->snap_mask & SNAP_PLANE)
+        render_grid(rend, goxel->plane, goxel->grid_color, goxel->image->box);
     if (!box_is_null(goxel->image->box))
         render_box(rend, goxel->image->box, goxel->image_box_color,
-                   EFFECT_SEE_BACK);
+                   EFFECT_SEE_BACK | EFFECT_GRID);
     if (goxel->show_export_viewport)
-        render_export_viewport(goxel, viewport);
+        render_export_viewport(viewport);
+
+    // XXX: cleanup this!
+    int rect[4] = {(int)viewport[0],
+                   (int)(goxel->screen_size[1] - viewport[1] - viewport[3]),
+                   (int)viewport[2],
+                   (int)viewport[3]};
+    render_submit(&goxel->rend, rect, goxel->back_color);
 }
 
 void image_update(image_t *img);
-void goxel_update_meshes(goxel_t *goxel, int mask)
+void goxel_update_meshes(int mask)
 {
     layer_t *layer;
     mesh_t *mesh;
@@ -644,10 +754,11 @@ void goxel_render_to_buf(uint8_t *buf, int w, int h, int bpp)
     uint8_t clear_color[4] = {0, 0, 0, 0};
 
     camera.aspect = (float)w / h;
+    camera_update(&camera);
+
     mesh = goxel->layers_mesh;
     fbo = texture_new_buffer(w * 2, h * 2, TF_DEPTH);
 
-    camera_update(&camera);
     mat4_copy(camera.view_mat, rend.view_mat);
     mat4_copy(camera.proj_mat, rend.proj_mat);
     rend.fbo = fbo->framebuffer;
@@ -655,7 +766,7 @@ void goxel_render_to_buf(uint8_t *buf, int w, int h, int bpp)
 
     render_mesh(&rend, mesh, 0);
     render_submit(&rend, rect, clear_color);
-    tmp_buf = calloc(w * h * bpp , bpp);
+    tmp_buf = calloc(w * h * 4, bpp);
     texture_get_data(fbo, w * 2, h * 2, bpp, tmp_buf);
     img_downsample(tmp_buf, w * 2, h * 2, bpp, buf);
     free(tmp_buf);
@@ -688,7 +799,7 @@ ACTION_REGISTER(export_as,
 
 
 // XXX: we could merge all the set_xxx_text function into a single one.
-void goxel_set_help_text(goxel_t *goxel, const char *msg, ...)
+void goxel_set_help_text(const char *msg, ...)
 {
     va_list args;
     free(goxel->help_text);
@@ -699,7 +810,7 @@ void goxel_set_help_text(goxel_t *goxel, const char *msg, ...)
     va_end(args);
 }
 
-void goxel_set_hint_text(goxel_t *goxel, const char *msg, ...)
+void goxel_set_hint_text(const char *msg, ...)
 {
     va_list args;
     free(goxel->hint_text);
@@ -710,7 +821,7 @@ void goxel_set_hint_text(goxel_t *goxel, const char *msg, ...)
     va_end(args);
 }
 
-void goxel_import_image_plane(goxel_t *goxel, const char *path)
+void goxel_import_image_plane(const char *path)
 {
     layer_t *layer;
     texture_t *tex;
@@ -723,7 +834,7 @@ void goxel_import_image_plane(goxel_t *goxel, const char *path)
     mat4_iscale(layer->mat, layer->image->w, layer->image->h, 1);
 }
 
-static int search_action_for_format_cb(const action_t *a, void *user)
+static int search_action_for_format_cb(action_t *a, void *user)
 {
     const char *path = USER_GET(user, 0);
     const char *type = USER_GET(user, 1);
@@ -739,7 +850,7 @@ static int goxel_import_file(const char *path)
 {
     const action_t *a = NULL;
     if (str_endswith(path, ".gox")) {
-        load_from_file(goxel, path);
+        load_from_file(path);
         return 0;
     }
     actions_iter(search_action_for_format_cb, USER_PASS(path, "import_", &a));
@@ -814,7 +925,7 @@ static void fill_selection(layer_t *layer)
     if (box_is_null(goxel->selection)) return;
     layer = layer ?: goxel->image->active_layer;
     mesh_op(layer->mesh, &goxel->painter, goxel->selection);
-    goxel_update_meshes(goxel, -1);
+    goxel_update_meshes(-1);
 }
 
 ACTION_REGISTER(fill_selection,
@@ -824,21 +935,6 @@ ACTION_REGISTER(fill_selection,
     .flags = ACTION_TOUCH_IMAGE,
 )
 
-static int show_grid_action(const action_t *a, astack_t *s)
-{
-    if (stack_type(s, 0) == 'b')
-        goxel->plane_hidden = !stack_get_b(s, 0);
-    stack_push_b(s, !goxel->plane_hidden);
-    return 0;
-}
-
-ACTION_REGISTER(grid_visible,
-    .help = "Show the grid",
-    .func = show_grid_action,
-    .shortcut = "#",
-    .flags = ACTION_TOGGLE,
-)
-
 static void copy_action(void)
 {
     painter_t painter;
@@ -881,7 +977,7 @@ ACTION_REGISTER(copy,
     .help = "Copy",
     .cfunc = copy_action,
     .csig = "v",
-    .shortcut = "Ctrl C",
+    .default_shortcut = "Ctrl C",
     .flags = 0,
 )
 
@@ -889,7 +985,7 @@ ACTION_REGISTER(past,
     .help = "Past",
     .cfunc = past_action,
     .csig = "v",
-    .shortcut = "Ctrl V",
+    .default_shortcut = "Ctrl V",
     .flags = ACTION_TOUCH_IMAGE,
 )
 
@@ -901,49 +997,54 @@ static int view_default(const action_t *a, astack_t *s)
     quat_set_identity(goxel->camera.rot);
     quat_irotate(goxel->camera.rot, -M_PI / 4, 1, 0, 0);
     quat_irotate(goxel->camera.rot, -M_PI / 4, 0, 0, 1);
-    goxel_update_meshes(goxel, -1);
+    goxel_update_meshes(-1);
     return 0;
 }
 
 static int view_set(const action_t *a, astack_t *s)
 {
     quat_copy(a->data, goxel->camera.rot);
-    goxel_update_meshes(goxel, -1);
+    goxel_update_meshes(-1);
     return 0;
 }
 
 ACTION_REGISTER(view_left,
     .help = "Set camera view to left",
+    .flags = ACTION_CAN_EDIT_SHORTCUT,
     .func = view_set,
     .data = &QUAT(0.5, -0.5, 0.5, 0.5),
-    .shortcut = "Ctrl 3",
+    .default_shortcut = "Ctrl 3",
 )
 
 ACTION_REGISTER(view_right,
     .help = "Set camera view to right",
+    .flags = ACTION_CAN_EDIT_SHORTCUT,
     .func = view_set,
     .data = &QUAT(-0.5, 0.5, 0.5, 0.5),
-    .shortcut = "3",
+    .default_shortcut = "3",
 )
 
 ACTION_REGISTER(view_top,
     .help = "Set camera view to top",
+    .flags = ACTION_CAN_EDIT_SHORTCUT,
     .func = view_set,
     .data = &QUAT(1, 0, 0, 0),
-    .shortcut = "7",
+    .default_shortcut = "7",
 )
 
 ACTION_REGISTER(view_default,
     .help = "Set camera view to default",
+    .flags = ACTION_CAN_EDIT_SHORTCUT,
     .func = view_default,
-    .shortcut = "5",
+    .default_shortcut = "5",
 )
 
 ACTION_REGISTER(view_front,
     .help = "Set camera view to front",
+    .flags = ACTION_CAN_EDIT_SHORTCUT,
     .func = view_set,
     .data = &QUAT(HS2, -HS2, 0, 0),
-    .shortcut = "1",
+    .default_shortcut = "1",
 )
 
 static void quit(void)
@@ -952,9 +1053,10 @@ static void quit(void)
 }
 ACTION_REGISTER(quit,
     .help = "Quit the application",
+    .flags = ACTION_CAN_EDIT_SHORTCUT,
     .cfunc = quit,
     .csig = "v",
-    .shortcut = "Ctrl Q",
+    .default_shortcut = "Ctrl Q",
 )
 
 static void undo(void) { image_undo(goxel->image); }
@@ -962,14 +1064,36 @@ static void redo(void) { image_redo(goxel->image); }
 
 ACTION_REGISTER(undo,
     .help = "Undo",
+    .flags = ACTION_CAN_EDIT_SHORTCUT,
     .cfunc = undo,
     .csig = "v",
-    .shortcut = "Ctrl Z",
+    .default_shortcut = "Ctrl Z",
+    .icon = ICON_ARROW_BACK,
 )
 
 ACTION_REGISTER(redo,
     .help = "Redo",
+    .flags = ACTION_CAN_EDIT_SHORTCUT,
     .cfunc = redo,
     .csig = "v",
-    .shortcut = "Ctrl Y",
+    .default_shortcut = "Ctrl Y",
+    .icon = ICON_ARROW_FORWARD,
+)
+
+static void toggle_mode(void)
+{
+    int mode = goxel->painter.mode;
+    switch (mode) {
+    case MODE_OVER:     mode = MODE_SUB; break;
+    case MODE_SUB:      mode = MODE_PAINT; break;
+    case MODE_PAINT:    mode = MODE_OVER; break;
+    }
+    goxel->painter.mode = mode;
+}
+
+ACTION_REGISTER(toggle_mode,
+    .help = "Toggle the tool mode (add, sub, paint)",
+    .flags = ACTION_CAN_EDIT_SHORTCUT,
+    .cfunc = toggle_mode,
+    .csig = "v",
 )
