@@ -36,6 +36,7 @@
 #include "noc_file_dialog.h"
 #include "block_def.h"
 #include "mesh.h"
+#include "theme.h"
 #include <float.h>
 #include <stdbool.h>
 #include <stdlib.h>
@@ -43,7 +44,7 @@
 #include <string.h>
 #include <stdarg.h>
 
-#define GOXEL_VERSION_STR "0.7.3"
+#define GOXEL_VERSION_STR "0.8.0"
 #define GOXEL_DEFAULT_THEME "original"
 
 // #### Set the DEBUG macro ####
@@ -54,6 +55,11 @@
 #       define DEBUG 0
 #   endif
 #endif
+
+#if !DEBUG && !defined(NDEBUG)
+#   define NDEBUG
+#endif
+#include <assert.h>
 // #############################
 
 
@@ -153,7 +159,7 @@ enum {
 #if DEBUG
 #  define GL(line) ({                                                   \
        line;                                                            \
-       if (check_gl_errors(__FILE__, __LINE__)) assert(false);          \
+       if (gl_check_errors(__FILE__, __LINE__)) assert(false);          \
    })
 #else
 #  define GL(line) line
@@ -167,70 +173,139 @@ enum {
 
 // Some useful inline functions / macros
 
+/*
+ * Macro: ARRAY_SIZE
+ * Return the number of elements in an array
+ */
 #define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
+
+/*
+ * Macro: SWAP
+ * Swap two variables
+ */
 #define SWAP(x0, x) do {typeof(x0) tmp = x0; x0 = x; x = tmp;} while (0)
 
-// Used to pass and get values to callback 'void *user' arguments.
+/*
+ * Macro: USER_PASS
+ * Used to pass values to callback 'void *user' arguments.
+ *
+ * For a function with the following declaration: f(void *user), we can pass
+ * several arguments packed in an array like that:
+ *
+ * int x, y;
+ * f(USER_PASS(&x, &y));
+ */
 #define USER_PASS(...) ((const void*[]){__VA_ARGS__})
+
+/*
+ * Macro: USER_GET
+ * Used to unpack values passed to a callback with USER_PASS:
+ *
+ * void f(void *user)
+ * {
+ *     char *arg1 = USER_GET(user, 0);
+ *     int   arg2 = *((int*)USER_GET(user, 1));
+ * }
+ */
 #define USER_GET(var, n) (((void**)var)[n])
 
-// IS_IN(x, ...): returns true if x is equal to any of the other arguments.
-#define IS_IN(x, ...) ({ \
-        bool _ret = false; \
-        const typeof(x) _V[] = {__VA_ARGS__}; \
-        int _i; \
-        for (_i = 0; _i < (int)ARRAY_SIZE(_V); _i++) \
-            if (x == _V[_i]) _ret = true; \
-        _ret; \
-    })
-
-// Some useful vector macros.
+/*
+ * Macro: vec3_set
+ * Set a 3 sized array values.
+ */
 #define vec3_set(v, x, y, z) do { \
     (v)[0] = (x); (v)[1] = (y); (v)[2] = (z); } while(0)
 
+/*
+ * Macro: vec4_set
+ * Set a 4 sized array values.
+ */
 #define vec4_set(v, x, y, z, w) do { \
     (v)[0] = (x); (v)[1] = (y); (v)[2] = (z); (v)[3] = (w); } while(0)
 
+/*
+ * Macro: vec4_copy
+ * Copy a 4 sized array into an other one.
+ */
 #define vec4_copy(a, b) do { \
         (b)[0] = (a)[0]; (b)[1] = (a)[1]; (b)[2] = (a)[2]; (b)[3] = (a)[3]; \
     } while (0)
 
+/*
+ * Macro: vec4_equal
+ * Test whether two 4 sized vectors are equal.
+ */
 #define vec4_equal(a, b) ({ \
         (a)[0] == (b)[0] && \
         (a)[1] == (b)[1] && \
         (a)[2] == (b)[2] && \
         (a)[3] == (b)[3]; })
 
-// Convertion between radian and degree.
+/* Define: DR2D
+ * Convertion ratio from radian to degree. */
 #define DR2D (180 / M_PI)
+
+/* Define: DR2D
+ * Convertion ratio from degree to radian. */
 #define DD2R (M_PI / 180)
 
+/* Define: KB
+ * 1024 */
 #define KB 1024
+
+/* Define: MB
+ * 1024^2 */
 #define MB (1024 * KB)
+
+/* Define: GB
+ * 1024^3 */
 #define GB (1024 * MB)
 
+/*
+ * Macro: min
+ * Safe min function.
+ */
 #define min(a, b) ({ \
       __typeof__ (a) _a = (a); \
       __typeof__ (b) _b = (b); \
       _a < _b ? _a : _b; \
       })
 
+/*
+ * Macro: max
+ * Safe max function.
+ */
 #define max(a, b) ({ \
       __typeof__ (a) _a = (a); \
       __typeof__ (b) _b = (b); \
       _a > _b ? _a : _b; \
       })
 
+/*
+ * Macro: max3
+ * Safe max function, return the max of three values.
+ */
 #define max3(x, y, z) (max((x), max((y), (z))))
+
+/*
+ * Macro: min3
+ * Safe max function, return the max of three values.
+ */
 #define min3(x, y, z) (min((x), min((y), (z))))
 
+/*
+ * Macro: clamp
+ * Clamp a value.
+ */
 #define clamp(x, a, b) (min(max(x, a), b))
 
-#define sign(x) ({ \
-      __typeof__ (x) _x = (x); \
-      (_x > 0) ? +1 : (_x < 0)? -1 : 0; \
-      })
-
+/*
+ * Macro: cmp
+ * Compare two values.
+ *
+ * Return:
+ *   +1 if a > b, -1 if a < b, 0 if a == b.
+ */
 #define cmp(a, b) ({ \
     __typeof__ (a) _a = (a); \
     __typeof__ (b) _b = (b); \
@@ -253,6 +328,12 @@ static inline float smoothstep(float edge0, float edge1, float x)
     return x * x * (3.0f - 2.0f * x);
 }
 
+/*
+ * Function: mix
+ * Linear blend of x and y.
+ *
+ * Similar to GLES mix function.
+ */
 static inline float mix(float x, float y, float t)
 {
     return (1.0 - t) * x + t * y;
@@ -271,69 +352,117 @@ static inline void set_flag(int *x, int flag, bool v)
     v ? (*x |= flag) : (*x &= ~flag);
 }
 
-// #############################
-
-// XXX: I should clean up a but the code of vec.h so that I can put those on
-// top.
-#include "box.h"
-#include "plane.h"
-
-
-// #### Utils ##################
-
+/*
+ * Function: read_file
+ * Read a file from disk.
+ *
+ * Return:
+ *   A newly allocated buffer containing the file data.
+ */
 char *read_file(const char *path, int *size);
 
 // Used internally by the LOG macro
 void dolog(int level, const char *msg,
            const char *func, const char *file, int line, ...);
-int check_gl_errors(const char *file, int line);
 
-// Enum of all the gl extensions we care for.
-enum {
-    GOX_GL_QCOM_tiled_rendering,
-    GOX_GL_OES_packed_depth_stencil,
-    GOX_GL_OES_depth_texture,
-    GOX_GL_EXT_discard_framebuffer,
-
-    GOX_GL_EXTENSIONS_COUNT
-};
-bool _has_gl_extension(int extension);
-#define has_gl_extension(x) (_has_gl_extension(GOX_##x))
-
-int create_program(const char *vertex_shader, const char *fragment_shader,
-                   const char *include);
-void delete_program(int prog);
+/*
+ * Function: img_read
+ * Read an image from a file.
+ */
 uint8_t *img_read(const char *path, int *width, int *height, int *bpp);
+
+/*
+ * Function: img_read_from_mem
+ * Read an image from memory.
+ */
 uint8_t *img_read_from_mem(const char *data, int size,
                            int *w, int *h, int *bpp);
+
+/*
+ * Function: img_write
+ * Write an image to a file.
+ */
 void img_write(const uint8_t *img, int w, int h, int bpp, const char *path);
+
+/*
+ * Function: img_write_to_mem
+ * Write an image to memory.
+ */
 uint8_t *img_write_to_mem(const uint8_t *img, int w, int h, int bpp,
                           int *size);
+
+/*
+ * Function: img_downsample
+ * Downsample an image by half, using interpolation.
+ */
 void img_downsample(const uint8_t *img, int w, int h, int bpp,
                     uint8_t *out);
-bool str_endswith(const char *str, const char *end);
-bool str_startswith(const char *s1, const char *s2);
-// Get gregorian date from unix time.
+/*
+ * Function: unix_to_dtf
+ * Get gregorian date from unix time.
+ *
+ * Parameters:
+ *   t  - Unix time.
+ *   iy - Output year.
+ *   im - Output month (1 - 12).
+ *   id - Output day (1 - 31).
+ *   h  - Output hour.
+ *   m  - Output minute.
+ *   s  - Output seconds.
+ */
 int unix_to_dtf(double t, int *iy, int *im, int *id, int *h, int *m, int *s);
+
+/*
+ * Function: utf_16_to_8
+ * Convert a string encoded in utf_16 to utf_8.
+ *
+ * Parameters:
+ *   in16   - Input string in utf 16 encoding.
+ *   out8   - Output buffer that receive the utf8 string.
+ *   size8  - Size of the output buffer.
+ */
 int utf_16_to_8(const wchar_t *in16, char *out8, size_t size8);
 
+/*
+ * Function: str_equ
+ * Return whether two strings are equal.
+ */
 static inline bool str_equ(const char *s1, const char *s2) {
     return strcmp(s1, s2) == 0;
 }
 
-// List all the files in a directory.
-// flags: unused for the moment.
-// Return the number of directory found.
-int list_dir(const char *url, int flags, void *user,
-             int (*f)(int i, const char *path, void *user));
+/*
+ * Function: str_endswith
+ * Return whether a string ends with an other one.
+ */
+bool str_endswith(const char *str, const char *end);
 
-// Convert from screen coordinate to world coordinates.
-// Similar to gluUnproject.
+/*
+ * Function: str_startswith
+ * Return whether a string starts with an other one.
+ */
+bool str_startswith(const char *s1, const char *s2);
+
+
+/*
+ * Function: unproject
+ * Convert from screen coordinate to world coordinates.
+ *
+ * Similar to gluUnproject.
+ *
+ * Parameters:
+ *   win        - Windows coordinates to be mapped.
+ *   model      - Modelview matrix.
+ *   proj       - Projection matrix.
+ *   viewport   - Viewport rect (x, y, w, h).
+ *   out        - Output of the computed object coordinates.
+ */
 void unproject(const float win[3], const float model[4][4],
                const float proj[4][4], const float viewport[4],
                float out[3]);
 
-/* Function: b64_decode
+/*
+ * Function: b64_decode
  * Decode a base64 string
  *
  * Parameters:
@@ -346,24 +475,139 @@ void unproject(const float win[3], const float model[4][4],
  */
 int b64_decode(const char *src, void *dest);
 
+/*
+ * Function: crc64
+ * Compute crc64 hash
+ *
+ * Parameters:
+ *   crc    - Initial crc value.
+ *   s      - Data pointer.
+ *   len    - Data size.
+ *
+ * Return:
+ *   The new crc64 value.
+ */
+uint64_t crc64(uint64_t crc, const uint8_t *s, uint64_t len);
+
 // #############################
 
+// XXX: I should clean up a but the code of vec.h so that I can put those on
+// top.
+#include "box.h"
+#include "plane.h"
+
+// ######### Section: GL utils ############################
+
+int gl_check_errors(const char *file, int line);
+
+/*
+ * Function: gl_has_extension
+ * Check whether an OpenGL extension is available.
+ */
+bool gl_has_extension(const char *extension);
+
+/*
+ * Function: gl_create_prog
+ * Helper function to create an OpenGL program.
+ */
+int gl_create_prog(const char *vertex_shader, const char *fragment_shader,
+                   const char *include);
+
+/*
+ * Function: gl_create_prog
+ * Helper function to delete an OpenGL program.
+ */
+void gl_delete_prog(int prog);
+
+/*
+ * Function: gl_gen_fbo
+ * Helper function to generate an OpenGL framebuffer object with an
+ * associated texture.
+ *
+ * Parameters:
+ *   w          - Width of the fbo.
+ *   h          - Height of the fbo.
+ *   format     - GL_RGBA or GL_DEPTH_COMPONENT.
+ *   out_fbo    - The created fbo.
+ *   out_tex    - The created texture.
+ */
+int gl_gen_fbo(int w, int h, GLenum format, int msaa,
+               GLuint *out_fbo, GLuint *out_tex);
+
+/* ########################################################################
+ * Section: System
+ * Some system related functions.  All the things that might depend on the
+ * operating system, not relying on libc, should go there.
+ */
 
 
-// #### System #################
+/*
+ * Global structure that hold pointers to system functions.  Need to be
+ * set at init time.
+ */
+typedef struct {
+    void *user;
+    void (*set_window_title)(void *user, const char *title);
+} sys_callbacks_t;
+extern sys_callbacks_t sys_callbacks;
+
+/*
+ * Function: sys_log
+ * Write a log message to output.
+ *
+ * Note: this should never be called directly, use the LOG_ macros instead.
+ */
 void sys_log(const char *msg);
-// List all the files in a directory.
+
+/*
+ * Function: sys_list_dir
+ * List all the files in a directory.
+ *
+ * Parameters:
+ *   dir      - Path of the directory we want to list.
+ *   callback - Function called once per entry.
+ *   user     - User data passed to the callback.
+ *
+ * Return:
+ *   The number of files found.
+ */
 int sys_list_dir(const char *dir,
                  int (*callback)(const char *dir, const char *name,
                                  void *user),
                  void *user);
+
+/*
+ * Function: sys_get_user_dir
+ * Return the user config directory for goxel
+ *
+ * On linux, this should be $HOME/.config/goxel.
+ */
 const char *sys_get_user_dir(void);
+
+/*
+ * Function: sys_make_dir
+ * Create all the directories parent of a given file path if they do not
+ * exist yet.
+ *
+ * For example, sys_make_dir("/a/b/c.txt") will create /a/ and /a/b/.
+ */
 int sys_make_dir(const char *path);
 
 const char *sys_get_clipboard_text(void* user);
 void sys_set_clipboard_text(void *user, const char *text);
 GLuint sys_get_screen_framebuffer(void);
+
+/*
+ * Function: sys_get_time
+ * Return the unix time (seconds since Jan 01 1970).
+ */
 double sys_get_time(void); // Unix time.
+
+/*
+ * Function: sys_set_window_title
+ * Set the window title.
+ */
+void sys_set_window_title(const char *title);
 // #############################
 
 
@@ -391,7 +635,6 @@ enum {
 // Reresent a 2d texture.
 typedef struct texture texture_t;
 struct texture {
-    texture_t   *next;      // All the textures are in a global list.
     int         ref;        // For reference copy.
     char        *path;      // Only for image textures.
 
@@ -448,9 +691,10 @@ bool  stack_get_b(const astack_t *s, int i);
 void  stack_pop(astack_t *s);
 
 enum {
-    ACTION_TOUCH_IMAGE    = 1 << 0,  // Push the undo history.
+    ACTION_TOUCH_IMAGE          = 1 << 0,  // Push the undo history.
     // Toggle actions accept and return a boolean value.
-    ACTION_TOGGLE         = 1 << 1,
+    ACTION_TOGGLE               = 1 << 1,
+    ACTION_CAN_EDIT_SHORTCUT    = 1 << 2,
 };
 
 // Represent an action.
@@ -459,7 +703,8 @@ struct action {
     const char      *id;            // Globally unique id.
     const char      *help;          // Help text.
     int             flags;
-    const char      *shortcut;      // Optional shortcut.
+    const char      *default_shortcut;
+    char            shortcut[8];    // Can be changed at runtime.
     int             icon;           // Optional icon id.
     int             (*func)(const action_t *a, astack_t *s);
     void            *data;
@@ -476,10 +721,10 @@ struct action {
 };
 
 void action_register(const action_t *action);
-const action_t *action_get(const char *id);
+action_t *action_get(const char *id);
 int action_exec(const action_t *action, const char *sig, ...);
 int action_execv(const action_t *action, const char *sig, va_list ap);
-void actions_iter(int (*f)(const action_t *action, void *user), void *user);
+void actions_iter(int (*f)(action_t *action, void *user), void *user);
 
 // Convenience macro to call action_exec directly from an action id.
 #define action_exec2(id, sig, ...) \
@@ -502,34 +747,57 @@ enum {
     ICON_NULL = 0,
 
     ICON_TOOL_BRUSH = 1,
-    ICON_TOOL_SHAPE = 2,
-    ICON_TOOL_LASER = 3,
+    ICON_TOOL_PICK = 2,
+    ICON_TOOL_SHAPE = 3,
     ICON_TOOL_PLANE = 4,
-    ICON_TOOL_MOVE = 5,
-    ICON_TOOL_PICK = 6,
-    ICON_TOOL_SELECTION = 7,
-    ICON_TOOL_PROCEDURAL = 8,
+    ICON_TOOL_LASER = 5,
+    ICON_TOOL_MOVE = 6,
+    ICON_TOOL_EXTRUDE = 7,
 
     ICON_MODE_ADD = 9,
     ICON_MODE_SUB = 10,
     ICON_MODE_PAINT = 11,
-    ICON_TOOL_EXTRUDE = 12,
+    ICON_SHAPE_CUBE = 12,
+    ICON_SHAPE_SPHERE = 13,
+    ICON_SHAPE_CYLINDER = 14,
+    ICON_TOOL_SELECTION = 15,
 
-    ICON_SHAPE_SPHERE = 17,
-    ICON_SHAPE_CUBE = 18,
-    ICON_SHAPE_CYLINDER = 19,
+    ICON_ADD = 17,
+    ICON_REMOVE = 18,
+    ICON_ARROW_BACK = 19,
+    ICON_ARROW_FORWARD = 20,
+    ICON_LINK = 21,
+    ICON_MENU = 22,
+    ICON_DELETE = 23,
+    ICON_TOOL_PROCEDURAL = 24,
 
-    ICON_ADD = 25,
-    ICON_REMOVE = 26,
+    ICON_VISIBILITY = 25,
+    ICON_VISIBILITY_OFF = 26,
     ICON_ARROW_DOWNWARD = 27,
     ICON_ARROW_UPWARD = 28,
-    ICON_VISIBILITY = 29,
-    ICON_VISIBILITY_OFF = 30,
-    ICON_EDIT = 31,
-    ICON_LINK = 32,
+
+    ICON_TOOLS = 33,
+    ICON_PALETTE = 34,
+    ICON_LAYERS = 35,
+    ICON_RENDER = 36,
+    ICON_CAMERA = 37,
+    ICON_IMAGE = 38,
+    ICON_EXPORT = 39,
+    ICON_DEBUG = 40,
+
+    ICON_VIEW = 41,
+    ICON_MATERIAL = 42,
 };
 
 
+/*
+ * Some icons have there color blended depending on the style.  We define
+ * them with a range in the icons atlas:
+ */
+#define ICON_COLORIZABLE_START 17
+#define ICON_COLORIZABLE_END   33
+
+
 // #### Tool/Operation/Painter #
 
 enum {
@@ -569,9 +837,6 @@ extern shape_t shape_cylinder;
 // The block size can only be 16.
 #define BLOCK_SIZE 16
 #define VOXEL_TEXTURE_SIZE 8
-// Number of sub position per voxel in the marching
-// cube rendering.
-#define MC_VOXEL_SUB_POS 8 // XXX: try to make it higher (up to 16!)
 
 // Structure used for the OpenGL array data of blocks.
 // XXX: we can probably make it smaller.
@@ -626,6 +891,7 @@ typedef struct painter {
     uint8_t         color[4];
     float           smoothness;
     int             symmetry; // bitfield X Y Z
+    float           symmetry_origin[3];
     float           (*box)[4][4];     // Clipping box (can be null)
 } painter_t;
 
@@ -689,11 +955,39 @@ int mesh_select(const mesh_t *mesh,
                             void *user),
                 void *user, mesh_t *selection);
 
-void mesh_merge(mesh_t *mesh, const mesh_t *other, int op,
+/*
+ * Function: mesh_merge
+ * Merge a mesh into an other using a given blending function.
+ *
+ * Parameters:
+ *   mesh   - The destination mesh we merge into.
+ *   other  - The source mesh we merge.  Unchanged by this function.
+ *   mode   - The blending function used.  One of the <MODE> enum values.
+ *   color  - A color to apply to the source mesh before merging.  Can be
+ *            set to NULL.
+ */
+void mesh_merge(mesh_t *mesh, const mesh_t *other, int mode,
                 const uint8_t color[4]);
 
+/*
+ * Function: mesh_generate_vertices
+ * Generate a vertice array for rendering a mesh block.
+ *
+ * Parameters:
+ *   mesh       - Input mesh.
+ *   block_pos  - Position of the mesh block to render.
+ *   effects    - Effect flags.
+ *   out        - Output array.
+ *   size       - Output the size of a single face.
+ *                4 for quads and 3 for triangles.  Normal mesh uses quad
+ *                but marching cube effect return triangle arrays.
+ *   subdivide  - Ouput the number of subdivisions used for a voxel.  Normal
+ *                render uses 1 unit per voxel, but marching cube rendering
+ *                can use more.
+ */
 int mesh_generate_vertices(const mesh_t *mesh, const int block_pos[3],
-                           int effects, voxel_vertex_t *out);
+                           int effects, voxel_vertex_t *out,
+                           int *size, int *subdivide);
 
 // XXX: use int[2][3] for the box?
 void mesh_crop(mesh_t *mesh, const float box[4][4]);
@@ -723,6 +1017,10 @@ enum {
     EFFECT_NO_SHADING       = 1 << 10,
     EFFECT_STRIP            = 1 << 11,
     EFFECT_WIREFRAME        = 1 << 12,
+    EFFECT_GRID             = 1 << 13,
+
+    EFFECT_PROJ_SCREEN      = 1 << 14, // Image project in screen.
+    EFFECT_ANTIALIASING     = 1 << 15,
 };
 
 typedef struct {
@@ -760,8 +1058,8 @@ struct renderer
 void render_init(void);
 void render_deinit(void);
 void render_mesh(renderer_t *rend, const mesh_t *mesh, int effects);
-void render_plane(renderer_t *rend, const float plane[4][4],
-                  const uint8_t color[4]);
+void render_grid(renderer_t *rend, const float plane[4][4],
+                 const uint8_t color[4], const float clip_box[4][4]);
 void render_line(renderer_t *rend, const float a[3], const float b[3],
                  const uint8_t color[4]);
 void render_box(renderer_t *rend, const float box[4][4],
@@ -769,6 +1067,18 @@ void render_box(renderer_t *rend, const float box[4][4],
 void render_sphere(renderer_t *rend, const float mat[4][4]);
 void render_img(renderer_t *rend, texture_t *tex, const float mat[4][4],
                 int efffects);
+
+/*
+ * Function: render_img2
+ * Render an image directly from it's pixel data.
+ *
+ * XXX: this is experimental: eventually I think we should remove render_img
+ * and only user render_img2 (renamed to render_img).
+ */
+void render_img2(renderer_t *rend,
+                 const uint8_t *data, int w, int h, int bpp,
+                 const float mat[4][4], int effects);
+
 void render_rect(renderer_t *rend, const float plane[4][4], int effects);
 // Flushes all the queued render items.  Actually calls opengl.
 //  rect: the viewport rect (passed to glViewport).
@@ -784,13 +1094,17 @@ void render_get_light_dir(const renderer_t *rend, float out[3]);
 void render_get_block_pos(renderer_t *rend, const mesh_t *mesh,
                           int id, int pos[3]);
 
-// #############################
 
+/* ##############################
+ * Section: Model3d
+ *
+ * Functions to render 3d vertex models, like cube, plane, etc.
+ */
 
-
-
-// #### Model3d ################
-
+/*
+ * Type: model_vertex_t
+ * Container for a single vertex shader data.
+ */
 typedef struct {
      float    pos[3]    __attribute__((aligned(4)));
      float    normal[3] __attribute__((aligned(4)));
@@ -798,6 +1112,10 @@ typedef struct {
      float    uv[2]     __attribute__((aligned(4)));
 } model_vertex_t;
 
+/*
+ * Type: model3d_t
+ * Define a 3d model.
+ */
 typedef struct {
     int              nb_vertices;
     model_vertex_t   *vertices;
@@ -811,19 +1129,66 @@ typedef struct {
     bool    dirty;
 } model3d_t;
 
+/*
+ * Function: model3d_init
+ * Should be called once before any mode3d functions.
+ */
 void model3d_init(void);
+
+/*
+ * Function: model3d_cube
+ * Create a 3d cube from (0, 0, 0) to (1, 1, 1)
+ */
 model3d_t *model3d_cube(void);
+
+/*
+ * Function: model3d_wire_cube
+ * Create a 3d wire cube from (0, 0, 0) to (1, 1, 1)
+ */
 model3d_t *model3d_wire_cube(void);
+
+/*
+ * Function: model3d_sphere
+ * Create a sphere of radius 1, centered at (0, 0).
+ */
 model3d_t *model3d_sphere(int slices, int stacks);
+
+/*
+ * Function: model3d_grid
+ * Create a grid plane.
+ */
 model3d_t *model3d_grid(int nx, int ny);
+
+/*
+ * Function: model3d_line
+ * Create a line from (-0.5, 0, 0) to (+0.5, 0, 0).
+ */
 model3d_t *model3d_line(void);
+
+/*
+ * Function: model3d_line
+ * Create a 2d rect on xy plane, of size 1x1 centered on the origin.
+ */
 model3d_t *model3d_rect(void);
+
+/*
+ * Function: model3d_line
+ * Create a 2d rect on xy plane, of size 1x1 centered on the origin.
+ */
 model3d_t *model3d_wire_rect(void);
+
+/*
+ * Function: model3d_render
+ * Render a 3d model using OpenGL calls.
+ */
 void model3d_render(model3d_t *model3d,
-                    const float model[4][4], const float proj[4][4],
+                    const float model[4][4],
+                    const float view[4][4],
+                    const float proj[4][4],
                     const uint8_t color[4],
                     const texture_t *tex,
                     const float light[3],
+                    const float clip_box[4][4],
                     int   effects);
 
 // #### Palette ################
@@ -908,16 +1273,47 @@ typedef struct inputs
 
 
 
-// #### Mouse gestures #####################################3
+/* #########################################
+ * Section: Mouse gestures
+ * Detect 2d mouse or touch gestures.
+ *
+ * The way it works is that we have to create one instance of
+ * <gesture_t> per gesture we can recognise, and then use the
+ * <gesture_update> function to update their states and call the callback
+ * functions of active gestures.
+ */
 
+/* XXX: This part of the code is not very clear */
+
+/*
+ * Enum: GESTURE_TYPES
+ * Define the different types of recognised gestures.
+ *
+ * GESTURE_DRAG     - Click and drag the mouse.
+ * GESTURE_CLICK    - Single click.
+ * GESTURE_PINCH    - Two fingers pinch.
+ * GESTURE_HOVER    - Move the mouse without clicking.
+ */
 enum {
-    // Supported type of gestures.
     GESTURE_DRAG        = 1 << 0,
     GESTURE_CLICK       = 1 << 1,
     GESTURE_PINCH       = 1 << 2,
     GESTURE_HOVER       = 1 << 3,
+};
 
-    // Gesture states.
+/*
+ * Enum: GESTURE_STATES
+ * Define the states a gesture can be in.
+ *
+ * GESTURE_POSSIBLE     - The gesture is not recognised yet (default state).
+ * GESTURE_RECOGNISED   - The gesture has been recognised.
+ * GESTURE_BEGIN        - The gesture has begun.
+ * GESTURE_UPDATE       - The gesture is in progress.
+ * GESTURE_END          - The testure has ended.
+ * GESTURE_TRIGGERED    - For click gestures: the gesture has occured.
+ * GESTURE_FAILED       - The gesture has failed.
+ */
+enum {
     GESTURE_POSSIBLE = 0,
     GESTURE_RECOGNISED,
     GESTURE_BEGIN,
@@ -927,6 +1323,10 @@ enum {
     GESTURE_FAILED,
 };
 
+/*
+ * Type: gesture_t
+ * Structure used to handle a given gesture.
+ */
 typedef struct gesture gesture_t;
 struct gesture
 {
@@ -941,13 +1341,48 @@ struct gesture
     int     (*callback)(const gesture_t *gest, void *user);
 };
 
+/*
+ * Function: gesture_update
+ * Update the state of a list of gestures, and call the gestures
+ * callbacks as needed.
+ *
+ * Parameters:
+ *   nb         - Number of gestures.
+ *   gestures   - A pointer to an array of <gesture_t> instances.
+ *   inputs     - The inputs structure.
+ *   viewport   - Current viewport rect.
+ *   user       - User data pointer, passed to the callbacks.
+ */
 int gesture_update(int nb, gesture_t *gestures[],
                    const inputs_t *inputs, const float viewport[4],
                    void *user);
 
 
-// #############################
+/* #############################
+ * Section: Camera
+ * Camera manipulation functions.
+ */
 
+/* Type: camera_t
+ * Camera structure.
+ *
+ * The actual position of the camera is constructed from a distance, a
+ * rotation and an offset:
+ *
+ * Pos = ofs * rot * dist
+ *
+ * Attributes:
+ *   next, prev - Used for linked list of cameras in an image.
+ *   name       - Name to show in the GUI.
+ *   ortho      - Set to true for orthographic projection.
+ *   dist       - Distance used to compute the position.
+ *   rot        - Camera rotation quaternion.
+ *   ofs        - Lateral offset of the camera position.
+ *   fovy       - Field of view in y direction.
+ *   aspect     - Aspect ratio.
+ *   view_mat   - Modelview transformation matrix (auto computed).
+ *   proj_mat   - Projection matrix (auto computed).
+ */
 typedef struct camera camera_t;
 struct camera
 {
@@ -965,25 +1400,62 @@ struct camera
     float proj_mat[4][4];    // Proj transform from camera coordinates.
 };
 
+/*
+ * Function: camera_new
+ * Create a new camera.
+ *
+ * Parameters:
+ *   name - The name of the camera.
+ *
+ * Return:
+ *   A newly instanciated camera.
+ */
 camera_t *camera_new(const char *name);
+
+/*
+ * Function: camera_delete
+ * Delete a camera
+ */
 void camera_delete(camera_t *camera);
+
+/*
+ * Function: camera_set
+ * Set a camera position from an other camera.
+ */
 void camera_set(camera_t *camera, const camera_t *other);
+
+/*
+ * Function: camera_update
+ * Update camera matrices.
+ */
 void camera_update(camera_t *camera);
-// Adjust the camera settings so that the rotation works for a given
-// position.
+
+/*
+ * Function: camera_set_target
+ * Adjust the camera settings so that the rotation works for a given
+ * position.
+ */
 void camera_set_target(camera_t *camera, const float pos[3]);
 
-// Function: camera_get_ray
-// Get the raytracing ray of the camera at a given screen position.
-//
-// Parameters:
-//   win   - Pixel position in screen coordinates.
-//   view  - Viewport rect: [min_x, min_y, max_x, max_y].
-//   o     - Output ray origin.
-//   d     - Output ray direction.
+/*
+ * Function: camera_get_ray
+ * Get the raytracing ray of the camera at a given screen position.
+ *
+ * Parameters:
+ *   win   - Pixel position in screen coordinates.
+ *   view  - Viewport rect: [min_x, min_y, max_x, max_y].
+ *   o     - Output ray origin.
+ *   d     - Output ray direction.
+ */
 void camera_get_ray(const camera_t *camera, const float win[2],
                     const float viewport[4], float o[3], float d[3]);
 
+/*
+ * Function: camera_fit_box
+ * Move a camera so that a given box is entirely visible.
+ */
+void camera_fit_box(camera_t *camera, const float box[4][4]);
+
 typedef struct history history_t;
 
 typedef struct layer layer_t;
@@ -1074,6 +1546,8 @@ enum {
     CURSOR_PRESSED      = 1 << 0,
     CURSOR_SHIFT        = 1 << 1,
     CURSOR_CTRL         = 1 << 2,
+
+    CURSOR_OUT          = 1 << 3, // Outside of sensing area.
 };
 
 typedef struct gesture3d gesture3d_t;
@@ -1113,7 +1587,7 @@ struct tool {
     const char *action_id;
     int (*iter_fn)(tool_t *tool, const float viewport[4]);
     int (*gui_fn)(tool_t *tool);
-    const char *shortcut;
+    const char *default_shortcut;
     int state; // XXX: to be removed I guess.
     int flags;
 };
@@ -1134,15 +1608,12 @@ int tool_iter(tool_t *tool, const float viewport[4]);
 int tool_gui(tool_t *tool);
 
 int tool_gui_snap(void);
-int tool_gui_mode(void);
 int tool_gui_shape(void);
 int tool_gui_radius(void);
 int tool_gui_smoothness(void);
 int tool_gui_color(void);
 int tool_gui_symmetry(void);
 
-
-
 typedef struct goxel
 {
     int        screen_size[2];
@@ -1165,11 +1636,11 @@ typedef struct goxel
     float      snap_offset;  // Only for brush tool, remove that?
 
     float      plane[4][4];         // The snapping plane.
-    bool       plane_hidden;  // Set to true to hide the plane.
     bool       show_export_viewport;
 
     camera_t   camera;
 
+    bool       use_cycles; // Render with cycles.
     uint8_t    back_color[4];
     uint8_t    grid_color[4];
     uint8_t    image_box_color[4];
@@ -1182,6 +1653,7 @@ typedef struct goxel
 
     tool_t     *tool;
     float      tool_radius;
+    bool       no_edit; // Disable editing.
 
     // Some state for the tool iter functions.
     float      tool_plane[4][4];
@@ -1214,51 +1686,49 @@ typedef struct goxel
         gesture_t hover;
     } gestures;
 
+    // Hold info about the cycles rendering task.
+    struct {
+        int status;         // 0: stopped, 1: running, 2: finished.
+        uint8_t *buf;       // RGBA buffer.
+        int w, h;           // Size of the buffer.
+        char output[1024];  // Output path.
+        float progress;
+        bool force_restart;
+    } render_task;
+
 } goxel_t;
 
 // the global goxel instance.
 extern goxel_t *goxel;
 
-void goxel_init(goxel_t *goxel);
-void goxel_release(goxel_t *goxel);
-void goxel_iter(goxel_t *goxel, inputs_t *inputs);
-void goxel_render(goxel_t *goxel);
-void goxel_render_view(goxel_t *goxel, const float viewport[4]);
+// XXX: add some doc.
+void goxel_init(void);
+void goxel_release(void);
+void goxel_reset(void);
+int goxel_iter(inputs_t *inputs);
+void goxel_render(void);
+void goxel_render_view(const float viewport[4]);
+void goxel_render_export_view(const float viewport[4]);
 // Called by the gui when the mouse hover a 3D view.
 // XXX: change the name since we also call it when the mouse get out of
 // the view.
-void goxel_mouse_in_view(goxel_t *goxel, const float viewport[4],
-                         const inputs_t *inputs);
+void goxel_mouse_in_view(const float viewport[4], const inputs_t *inputs);
 
-int goxel_unproject(goxel_t *goxel, const float viewport[4],
-                    const float pos[2], int snap_mask, float offset,
-                    float out[3], float normal[3]);
-
-bool goxel_unproject_on_mesh(goxel_t *goxel, const float viewport[4],
-                     const float pos[2], mesh_t *mesh,
-                     float out[3], float normal[3]);
-
-bool goxel_unproject_on_plane(goxel_t *goxel, const float viewport[4],
-                     const float pos[2], const float plane[4][4],
-                     float out[3], float normal[3]);
-bool goxel_unproject_on_box(goxel_t *goxel, const float viewport[4],
-                     const float pos[2], const float box[4][4], bool inside,
-                     float out[3], float normal[3], int *face);
 // Recompute the meshes.  mask from MESH_ enum.
-void goxel_update_meshes(goxel_t *goxel, int mask);
+void goxel_update_meshes(int mask);
 
-void goxel_set_help_text(goxel_t *goxel, const char *msg, ...);
-void goxel_set_hint_text(goxel_t *goxel, const char *msg, ...);
+void goxel_set_help_text(const char *msg, ...);
+void goxel_set_hint_text(const char *msg, ...);
 
-void goxel_import_image_plane(goxel_t *goxel, const char *path);
+void goxel_import_image_plane(const char *path);
 
 // Render the view into an RGB[A] buffer.
 void goxel_render_to_buf(uint8_t *buf, int w, int h, int bpp);
 
 // #############################
 
-void save_to_file(goxel_t *goxel, const char *path, bool with_preview);
-int load_from_file(goxel_t *goxel, const char *path);
+void save_to_file(const char *path, bool with_preview);
+int load_from_file(const char *path);
 
 // Iter info of a gox file, without actually reading it.
 // For the moment only returns the image preview if available.
@@ -1272,84 +1742,16 @@ int gox_iter_infos(const char *path,
 void hsl_to_rgb(const uint8_t hsl[3], uint8_t rgb[3]);
 void rgb_to_hsl(const uint8_t rgb[3], uint8_t hsl[3]);
 
-// #### Gui ####################
-
-#define THEME_SIZES(X) \
-    X(item_height) \
-    X(icons_height) \
-    X(item_padding_h) \
-    X(item_rounding) \
-    X(item_spacing_h) \
-    X(item_spacing_v) \
-    X(item_inner_spacing_h)
-
-
-enum {
-    THEME_GROUP_BASE,
-    THEME_GROUP_WIDGET,
-    THEME_GROUP_TAB,
-    THEME_GROUP_MENU,
-    THEME_GROUP_COUNT
-};
-
-enum {
-    THEME_COLOR_BACKGROUND,
-    THEME_COLOR_OUTLINE,
-    THEME_COLOR_INNER,
-    THEME_COLOR_INNER_SELECTED,
-    THEME_COLOR_TEXT,
-    THEME_COLOR_TEXT_SELECTED,
-    THEME_COLOR_COUNT
-};
-
-typedef struct {
-    const char *name;
-    int parent;
-    bool colors[THEME_COLOR_COUNT];
-} theme_group_info_t;
-extern theme_group_info_t THEME_GROUP_INFOS[THEME_GROUP_COUNT];
-
-typedef struct {
-    const char *name;
-} theme_color_info_t;
-extern theme_color_info_t THEME_COLOR_INFOS[THEME_COLOR_COUNT];
-
-typedef struct {
-    uint8_t colors[THEME_COLOR_COUNT][4];
-} theme_group_t;
-
-typedef struct theme theme_t;
-struct theme {
-    char name[64];
-
-    struct {
-        int  item_height;
-        int  icons_height;
-        int  item_padding_h;
-        int  item_rounding;
-        int  item_spacing_h;
-        int  item_spacing_v;
-        int  item_inner_spacing_h;
-    } sizes;
-
-    theme_group_t groups[THEME_GROUP_COUNT];
-
-    theme_t *prev, *next; // Global list of themes.
-};
-
-// Return the current theme.
-theme_t *theme_get(void);
-theme_t *theme_get_list(void);
-void theme_revert_default(void);
-void theme_save(void);
-void theme_get_color(int group, int color, bool selected, uint8_t out[4]);
-void theme_set(const char *name);
+/* ################################
+ * Section: Gui
+ */
 
 void gui_release(void);
-void gui_iter(goxel_t *goxel, const inputs_t *inputs);
+void gui_iter(const inputs_t *inputs);
 void gui_render(void);
 
 // Gui widgets:
+bool gui_collapsing_header(const char *label);
 void gui_text(const char *label, ...);
 bool gui_button(const char *label, float w, int icon);
 void gui_group_begin(const char *label);
@@ -1377,6 +1779,14 @@ float gui_get_avail_width(void);
 void gui_same_line(void);
 void gui_enabled_begin(bool enabled);
 void gui_enabled_end(void);
+void gui_alert(const char *title, const char *msg);
+
+void gui_columns(int count);
+void gui_next_column(void);
+void gui_separator(void);
+
+void gui_push_id(const char *id);
+void gui_pop_id(void);
 
 enum {
     GUI_POPUP_FULL      = 1 << 0,
@@ -1454,6 +1864,12 @@ void sound_init(void);
 void sound_play(const char *sound);
 void sound_iter(void);
 
+// Section: cycles
+void cycles_init(void);
+void cycles_release(void);
+void cycles_render(uint8_t *buffer, int *w, int *h, const camera_t *cam,
+                   float *progress, bool force_restart);
+
 // Section: tests
 
 /* Function: tests_run
