@@ -18,18 +18,11 @@
 
 #include "goxel.h"
 
-#if DEBUG
-#   define DEBUG_ONLY(x) x
-#else
-#   define DEBUG_ONLY(x)
-#endif
-
 #ifdef GLES2
 #   define GLFW_INCLUDE_ES2
 #endif
 #include <GLFW/glfw3.h>
 
-static goxel_t      *g_goxel = NULL;
 static inputs_t     *g_inputs = NULL;
 static GLFWwindow   *g_window = NULL;
 static float        g_scale = 1;
@@ -136,8 +129,8 @@ static void loop_function(void) {
         glfwGetMouseButton(g_window, GLFW_MOUSE_BUTTON_MIDDLE) == GLFW_PRESS;
     g_inputs->touches[0].down[2] =
         glfwGetMouseButton(g_window, GLFW_MOUSE_BUTTON_RIGHT) == GLFW_PRESS;
-    goxel_iter(g_goxel, g_inputs);
-    goxel_render(g_goxel);
+    goxel_iter(g_inputs);
+    goxel_render();
 
     memset(g_inputs, 0, sizeof(*g_inputs));
     glfwSwapBuffers(g_window);
@@ -177,19 +170,28 @@ static void load_icon(GLFWimage *image, const char *path)
 
 static void set_window_icon(GLFWwindow *window)
 {
-    GLFWimage icons[3];
+    GLFWimage icons[7];
     int i;
     load_icon(&icons[0], "asset://data/icons/icon16.png");
-    load_icon(&icons[1], "asset://data/icons/icon32.png");
-    load_icon(&icons[2], "asset://data/icons/icon48.png");
-    glfwSetWindowIcon(window, 3, icons);
-    for (i = 0; i < 3; i++) free(icons[i].pixels);
+    load_icon(&icons[1], "asset://data/icons/icon24.png");
+    load_icon(&icons[2], "asset://data/icons/icon32.png");
+    load_icon(&icons[3], "asset://data/icons/icon48.png");
+    load_icon(&icons[4], "asset://data/icons/icon64.png");
+    load_icon(&icons[5], "asset://data/icons/icon128.png");
+    load_icon(&icons[6], "asset://data/icons/icon256.png");
+    glfwSetWindowIcon(window, 7, icons);
+    for (i = 0; i < 7; i++) free(icons[i].pixels);
 }
 
 #else
 static void set_window_icon(GLFWwindow *window) {}
 #endif
 
+static void set_window_title(void *user, const char *title)
+{
+    glfwSetWindowTitle(g_window, title);
+}
+
 int main(int argc, char **argv)
 {
     args_t args = {.scale = 1};
@@ -198,9 +200,10 @@ int main(int argc, char **argv)
     const GLFWvidmode *mode;
     int ret = 0;
     inputs_t inputs = {};
-    const char *title = "Goxel " GOXEL_VERSION_STR DEBUG_ONLY(" (debug)");
     g_inputs = &inputs;
-    g_goxel = calloc(1, sizeof(*g_goxel));
+
+    // Setup sys callbacks.
+    sys_callbacks.set_window_title = set_window_title;
 
 #ifndef NO_ARGP
     argp_parse (&argp, argc, argv, 0, 0, &args);
@@ -208,10 +211,10 @@ int main(int argc, char **argv)
     g_scale = args.scale;
 
     glfwInit();
-    glfwWindowHint(GLFW_SAMPLES, 2);
+    glfwWindowHint(GLFW_SAMPLES, 4);
     monitor = glfwGetPrimaryMonitor();
     mode = glfwGetVideoMode(monitor);
-    window = glfwCreateWindow(mode->width, mode->height, title, NULL, NULL);
+    window = glfwCreateWindow(mode->width, mode->height, "Goxel", NULL, NULL);
     g_window = window;
     glfwMakeContextCurrent(window);
     glfwSetScrollCallback(window, on_scroll);
@@ -222,9 +225,12 @@ int main(int argc, char **argv)
 #ifdef WIN32
     glewInit();
 #endif
-    goxel_init(g_goxel);
+    goxel_init();
     // Run the unit tests in debug.
-    if (DEBUG) tests_run();
+    if (DEBUG) {
+        tests_run();
+        goxel_reset();
+    }
 
     if (args.input)
         action_exec2("import", "p", args.input);
@@ -239,6 +245,6 @@ int main(int argc, char **argv)
     }
     start_main_loop(loop_function);
 end:
-    goxel_release(g_goxel);
+    goxel_release();
     return ret;
 }
