@@ -288,6 +288,7 @@ void mesh_op(mesh_t *mesh, const painter_t *painter, const float box[4][4])
     float box2[4][4];
     mesh_t *cached;
     static cache_t *cache = NULL;
+    const float *sym_o = painter->symmetry_origin;
 
     // Check if the operation has been cached.
     if (!cache) cache = cache_create(32);
@@ -312,9 +313,11 @@ void mesh_op(mesh_t *mesh, const painter_t *painter, const float box[4][4])
             if (!(painter->symmetry & (1 << i))) continue;
             painter2.symmetry &= ~(1 << i);
             mat4_set_identity(box2);
+            mat4_itranslate(box2, +sym_o[0], +sym_o[1], +sym_o[2]);
             if (i == 0) mat4_iscale(box2, -1,  1,  1);
             if (i == 1) mat4_iscale(box2,  1, -1,  1);
             if (i == 2) mat4_iscale(box2,  1,  1, -1);
+            mat4_itranslate(box2, -sym_o[0], -sym_o[1], -sym_o[2]);
             mat4_imul(box2, box);
             mesh_op(mesh, &painter2, box2);
         }
@@ -326,10 +329,13 @@ void mesh_op(mesh_t *mesh, const painter_t *painter, const float box[4][4])
     mat4_iscale(mat, 1 / size[0], 1 / size[1], 1 / size[2]);
     mat4_invert(mat, mat);
     use_box = painter->box && !box_is_null(*painter->box);
-    // XXX: cleanup.
-    skip_src_empty = IS_IN(mode, MODE_SUB, MODE_SUB_CLAMP, MODE_MULT_ALPHA);
-    skip_dst_empty = IS_IN(mode, MODE_SUB, MODE_SUB_CLAMP, MODE_MULT_ALPHA,
-                                 MODE_INTERSECT);
+    skip_src_empty = mode == MODE_SUB ||
+                     mode == MODE_SUB_CLAMP ||
+                     mode == MODE_MULT_ALPHA;
+    skip_dst_empty = mode == MODE_SUB ||
+                     mode == MODE_SUB_CLAMP ||
+                     mode == MODE_MULT_ALPHA ||
+                     mode == MODE_INTERSECT;
     if (mode != MODE_INTERSECT) {
         iter = mesh_get_box_iterator(mesh, box,
                 skip_dst_empty ? MESH_ITER_SKIP_EMPTY : 0);
@@ -346,8 +352,11 @@ void mesh_op(mesh_t *mesh, const painter_t *painter, const float box[4][4])
         if (use_box && !bbox_contains_vec(*painter->box, p)) continue;
         mat4_mul_vec3(mat, p, p);
         k = shape_func(p, size, painter->smoothness);
-        k = clamp(k / painter->smoothness, -1.0f, 1.0f);
-        v = k / 2.0f + 0.5f;
+        if (painter->smoothness) {
+            v = clamp(k / painter->smoothness, -1.0f, 1.0f) / 2.0f + 0.5f;
+        } else {
+            v = (k >= 0.f) ? 1.f : 0.f;
+        }
         if (!v && skip_src_empty) continue;
         memcpy(c, painter->color, 4);
         c[3] *= v;
@@ -386,18 +395,21 @@ static void block_merge(mesh_t *mesh, const mesh_t *other, const int pos[3],
 
     // XXX: cleanup this code!
 
-    if (IS_IN(mode, MODE_OVER, MODE_MAX, MODE_SUB, MODE_SUB_CLAMP)
-            && id2 == 0) {
+    if (    (mode == MODE_OVER ||
+             mode == MODE_MAX ||
+             mode == MODE_SUB ||
+             mode == MODE_SUB_CLAMP) && id2 == 0)
+    {
         return;
     }
 
-    if (IS_IN(mode, MODE_OVER, MODE_MAX) && id1 == 0 && !color) {
+    if ((mode == MODE_OVER || mode == MODE_MAX) && id1 == 0 && !color) {
         mesh_copy_block(other, pos, mesh, pos);
         return;
     }
 
-    if (IS_IN(mode, MODE_MULT_ALPHA) && id1 == 0) return;
-    if (IS_IN(mode, MODE_MULT_ALPHA) && id2 == 0) {
+    if ((mode == MODE_MULT_ALPHA) && id1 == 0) return;
+    if ((mode == MODE_MULT_ALPHA) && id2 == 0) {
         // XXX: could just delete the block.
     }
 
