@@ -56,6 +56,7 @@ struct render_item_t
         float           mat[4][4];
     };
     uint8_t         color[4];
+    float           clip_box[4][4];
     bool            proj_screen; // Render with a 2d proj.
     model3d_t       *model3d;
     texture_t       *tex;
@@ -64,6 +65,7 @@ struct render_item_t
     GLuint      vertex_buffer;
     int         size;           // 4 (quads) or 3 (triangles).
     int         nb_elements;    // Number of quads or triangle.
+    int         subdivide;      // Unit per voxel (usually 1).
 };
 
 // The buffered item hash table.  For the moment it is only used of the blocks.
@@ -80,20 +82,9 @@ static model3d_t *g_grid_model;
 static model3d_t *g_rect_model;
 static model3d_t *g_wire_rect_model;
 
-// All the shaders code is at the bottom of the file.
-static const char *VSHADER;
-static const char *FSHADER;
-static const char *POS_DATA_VSHADER;
-static const char *POS_DATA_FSHADER;
-static const char *SHADOW_MAP_VSHADER;
-static const char *SHADOW_MAP_FSHADER;
-static const char *BACKGROUND_VSHADER;
-static const char *BACKGROUND_FSHADER;
-
 typedef struct {
     // Those values are used as a key to identify the shader.
-    const char *vshader;
-    const char *fshader;
+    const char *path;
     const char *include;
 
     GLint prog;
@@ -290,17 +281,17 @@ static void init_bump_texture(void)
     free(data);
 }
 
-static void init_prog(prog_t *prog, const char *vshader, const char *fshader,
-                      const char *include)
+static void init_prog(prog_t *prog, const char *path, const char *include)
 {
     char include_full[256];
+    const char *code;
     int attr;
     sprintf(include_full, "#define VOXEL_TEXTURE_SIZE %d.0\n%s\n",
             VOXEL_TEXTURE_SIZE, include ?: "");
-    prog->vshader = vshader;
-    prog->fshader = fshader;
+    code = assets_get(path, NULL);
+    prog->path = path;
     prog->include = include;
-    prog->prog = create_program(vshader, fshader, include_full);
+    prog->prog = gl_create_prog(code, code, include_full);
     for (attr = 0; attr < ARRAY_SIZE(ATTRIBUTES); attr++) {
         GL(glBindAttribLocation(prog->prog, attr, ATTRIBUTES[attr].name));
     }
@@ -331,20 +322,17 @@ static void init_prog(prog_t *prog, const char *vshader, const char *fshader,
     GL(glUniform1i(prog->u_shadow_tex_l, 2));
 }
 
-static prog_t *get_prog(const char *vshader, const char *fshader,
-                        const char *include)
+static prog_t *get_prog(const char *path, const char *include)
 {
     int i;
     prog_t *p = NULL;
     for (i = 0; i < ARRAY_SIZE(g_progs); i++) {
         p = &g_progs[i];
-        if (!p->vshader) break;
-        if (p->vshader == vshader && p->fshader == fshader &&
-            p->include == include)
-            return p;
+        if (!p->path) break;
+        if (p->path == path && p->include == include) return p;
     }
     assert(i < ARRAY_SIZE(g_progs));
-    init_prog(p, vshader, fshader, include);
+    init_prog(p, path, include);
     return p;
 }
 
@@ -365,6 +353,8 @@ void render_init()
     GL(glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, g_index_buffer));
     GL(glBufferData(GL_ELEMENT_ARRAY_BUFFER, BATCH_QUAD_COUNT * 6 * 2,
                     index_array, GL_STATIC_DRAW));
+    GL(glEnable(GL_LINE_SMOOTH));
+
     free(index_array);
     init_border_texture();
     init_bump_texture();
@@ -385,7 +375,7 @@ void render_deinit(void)
     int i;
     for (i = 0; i < ARRAY_SIZE(g_progs); i++) {
         if (g_progs[i].prog)
-            delete_program(g_progs[i].prog);
+            gl_delete_prog(g_progs[i].prog);
     }
     memset(&g_progs, 0, sizeof(g_progs));
     GL(glDeleteBuffers(1, &g_index_buffer));
@@ -447,8 +437,8 @@ static render_item_t *get_item_for_block(
                 BLOCK_SIZE * BLOCK_SIZE * BLOCK_SIZE * 6 * 4,
                 sizeof(*g_vertices_buffer));
     item->nb_elements = mesh_generate_vertices(
-            mesh, block_pos, effects, g_vertices_buffer);
-    item->size = (effects & EFFECT_MARCHING_CUBES) ? 3 : 4;
+            mesh, block_pos, effects, g_vertices_buffer,
+            &item->size, &item->subdivide);
     if (item->nb_elements > BATCH_QUAD_COUNT) {
         LOG_W("Too many quads!");
         item->nb_elements = BATCH_QUAD_COUNT;
@@ -485,6 +475,7 @@ static void render_block_(renderer_t *rend, mesh_t *mesh,
         block_id_f[0] = ((block_id >> 0) & 0xff) / 255.0;
         GL(glUniform2fv(prog->u_block_id_l, 1, block_id_f));
     }
+    GL(glUniform1f(prog->u_pos_scale_l, 1.f / item->subdivide));
 
     for (attr = 0; attr < ARRAY_SIZE(ATTRIBUTES); attr++) {
         GL(glVertexAttribPointer(attr,
@@ -540,14 +531,14 @@ static void compute_shadow_map_box(
                 float rect[6])
 {
     const float POS[8][3] = {
-        {-0.5, -0.5, -0.5},
-        {+0.5, -0.5, -0.5},
-        {+0.5, -0.5, +0.5},
-        {-0.5, -0.5, +0.5},
-        {-0.5, +0.5, -0.5},
-        {+0.5, +0.5, -0.5},
-        {+0.5, +0.5, +0.5},
-        {-0.5, +0.5, +0.5}
+        {0, 0, 0},
+        {1, 0, 0},
+        {1, 0, 1},
+        {0, 0, 1},
+        {0, 1, 0},
+        {1, 1, 0},
+        {1, 1, 1},
+        {0, 1, 1},
     };
     const int N = BLOCK_SIZE;
 
@@ -559,8 +550,12 @@ static void compute_shadow_map_box(
 
     get_light_dir(rend, false, light_dir);
     mat4_lookat(view_mat, light_dir, VEC(0, 0, 0), VEC(0, 1, 0));
-    for (i = 0; i < 6; i++)
-        rect[i] = NAN;
+    rect[0] = +FLT_MAX;
+    rect[1] = -FLT_MAX;
+    rect[2] = +FLT_MAX;
+    rect[3] = -FLT_MAX;
+    rect[4] = +FLT_MAX;
+    rect[5] = -FLT_MAX;
 
     DL_FOREACH(rend->items, item) {
         if (item->type != ITEM_MESH) continue;
@@ -587,7 +582,6 @@ static void render_mesh_(renderer_t *rend, mesh_t *mesh, int effects,
     prog_t *prog;
     float model[4][4];
     int attr, block_pos[3], block_id;
-    float pos_scale = 1.0f;
     float light_dir[3];
     bool shadow = false;
     mesh_iterator_t iter;
@@ -595,17 +589,14 @@ static void render_mesh_(renderer_t *rend, mesh_t *mesh, int effects,
     mat4_set_identity(model);
     get_light_dir(rend, true, light_dir);
 
-    if (effects & EFFECT_MARCHING_CUBES)
-        pos_scale = 1.0 / MC_VOXEL_SUB_POS;
-
     if (effects & EFFECT_RENDER_POS)
-        prog = get_prog(POS_DATA_VSHADER, POS_DATA_FSHADER, NULL);
+        prog = get_prog("asset://data/shaders/pos_data.glsl", NULL);
     else if (effects & EFFECT_SHADOW_MAP)
-        prog = get_prog(SHADOW_MAP_VSHADER, SHADOW_MAP_FSHADER, NULL);
+        prog = get_prog("asset://data/shaders/shadow_map.glsl", NULL);
     else {
         shadow = rend->settings.shadow;
-        prog = get_prog(VSHADER, FSHADER,
-                        shadow ? "#define SHADOW" : NULL);
+        prog = get_prog("asset://data/shaders/mesh.glsl",
+                         shadow ? "#define SHADOW" : NULL);
     }
 
     GL(glEnable(GL_DEPTH_TEST));
@@ -651,7 +642,6 @@ static void render_mesh_(renderer_t *rend, mesh_t *mesh, int effects,
     GL(glUniform1f(prog->u_m_shi_l, rend->settings.shininess));
     GL(glUniform1f(prog->u_m_smo_l, rend->settings.smoothness));
     GL(glUniform1f(prog->u_bshadow_l, rend->settings.border_shadow));
-    GL(glUniform1f(prog->u_pos_scale_l, pos_scale));
 
     for (attr = 0; attr < ARRAY_SIZE(ATTRIBUTES); attr++)
         GL(glEnableVertexAttribArray(attr));
@@ -711,47 +701,47 @@ void render_mesh(renderer_t *rend, const mesh_t *mesh, int effects)
 
 static void render_model_item(renderer_t *rend, const render_item_t *item)
 {
-    float view[4][4];
     float proj[4][4];
-    float (*proj_mat)[4][4];
+    const float (*proj_mat)[4][4];
+    const float (*view_mat)[4][4];
     float light[3];
 
-    mat4_copy(rend->view_mat, view);
-    mat4_imul(view, item->mat);
-
     if (item->proj_screen) {
         mat4_ortho(proj, -0.5, +0.5, -0.5, +0.5, -10, +10);
         proj_mat = &proj;
-        mat4_copy(item->mat, view);
+        view_mat = &mat4_identity;
     } else {
         proj_mat = &rend->proj_mat;
+        view_mat = &rend->view_mat;
     }
 
     if (!(item->effects & EFFECT_WIREFRAME))
         get_light_dir(rend, false, light);
 
-    model3d_render(item->model3d, view, *proj_mat, item->color,
-                   item->tex, light, item->effects);
+    model3d_render(item->model3d,
+                   item->mat, *view_mat, *proj_mat,
+                   item->color,
+                   item->tex, light, item->clip_box, item->effects);
 }
 
 static void render_grid_item(renderer_t *rend, const render_item_t *item)
 {
     int x, y, n;
-    float view2[4][4], view3[4][4];
+    float model_mat[4][4];
 
-    mat4_copy(rend->view_mat, view2);
-    mat4_imul(view2, item->mat);
     n = 3;
     for (y = -n; y < n; y++)
     for (x = -n; x < n; x++) {
-        mat4_translate(view2, x + 0.5, y + 0.5, 0, view3);
-        model3d_render(item->model3d, view3, rend->proj_mat,
-                       item->color, NULL, NULL, 0);
+        mat4_copy(item->mat, model_mat);
+        mat4_translate(model_mat, x + 0.5, y + 0.5, 0, model_mat);
+        model3d_render(item->model3d,
+                       model_mat, rend->view_mat, rend->proj_mat,
+                       item->color, NULL, NULL, item->clip_box, 0);
     }
 }
 
-void render_plane(renderer_t *rend, const float plane[4][4],
-                  const uint8_t color[4])
+void render_grid(renderer_t *rend, const float plane[4][4],
+                 const uint8_t color[4], const float clip_box[4][4])
 {
     render_item_t *item = calloc(1, sizeof(*item));
     item->type = ITEM_GRID;
@@ -759,6 +749,7 @@ void render_plane(renderer_t *rend, const float plane[4][4],
     mat4_iscale(item->mat, 8, 8, 1);
     item->model3d = g_grid_model;
     copy_color(color, item->color);
+    if (clip_box) mat4_copy(clip_box, item->clip_box);
     DL_APPEND(rend->items, item);
 }
 
@@ -776,6 +767,26 @@ void render_img(renderer_t *rend, texture_t *tex, const float mat[4][4],
     DL_APPEND(rend->items, item);
 }
 
+void render_img2(renderer_t *rend,
+                 const uint8_t *data, int w, int h, int bpp,
+                 const float mat[4][4], int effects)
+{
+    // Experimental for the moment!
+    // Same as render_img, but we flip the texture!
+    render_item_t *item = calloc(1, sizeof(*item));
+    item->type = ITEM_MODEL3D;
+    mat ? mat4_copy(mat, item->mat) : mat4_set_identity(item->mat);
+    mat4_iscale(item->mat, 1, -1, 1);
+    item->proj_screen = effects & EFFECT_PROJ_SCREEN;
+    item->tex = texture_new_from_buf(data, w, h, bpp,
+            (effects & EFFECT_ANTIALIASING) ? 0 : TF_NEAREST);
+    item->model3d = g_rect_model;
+    copy_color(NULL, item->color);
+    item->effects = effects;
+    DL_APPEND(rend->items, item);
+}
+
+
 void render_rect(renderer_t *rend, const float plane[4][4], int effects)
 {
     render_item_t *item = calloc(1, sizeof(*item));
@@ -814,8 +825,8 @@ void render_box(renderer_t *rend, const float box[4][4],
                 const uint8_t color[4], int effects)
 {
     render_item_t *item = calloc(1, sizeof(*item));
-    assert((effects & (EFFECT_STRIP | EFFECT_WIREFRAME | EFFECT_SEE_BACK)) \
-            == effects);
+    assert((effects & (EFFECT_STRIP | EFFECT_WIREFRAME | EFFECT_SEE_BACK |
+                       EFFECT_GRID)) == effects);
     item->type = ITEM_MODEL3D;
     mat4_copy(box, item->mat);
     copy_color(color, item->color);
@@ -836,6 +847,7 @@ void render_sphere(renderer_t *rend, const float mat[4][4])
 
 static int item_sort_value(const render_item_t *a)
 {
+    if (a->effects & EFFECT_WIREFRAME) return 20;
     if (a->proj_screen)     return 10;
     switch (a->type) {
         case ITEM_MESH:     return 0;
@@ -847,7 +859,7 @@ static int item_sort_value(const render_item_t *a)
 
 static int item_sort_cmp(const render_item_t *a, const render_item_t *b)
 {
-    return sign(item_sort_value(a) - item_sort_value(b));
+    return cmp(item_sort_value(a), item_sort_value(b));
 }
 
 
@@ -919,7 +931,7 @@ static void render_background(renderer_t *rend, const uint8_t col[4])
         GL(glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT));
         return;
     }
-
+    GL(glClear(GL_DEPTH_BUFFER_BIT));
     // Add a small gradient to the color.
     vec4_set(c1, col[0] / 255., col[1] / 255., col[2] / 255., col[3] / 255.);
     vec4_set(c2, col[0] / 255., col[1] / 255., col[2] / 255., col[3] / 255.);
@@ -931,7 +943,7 @@ static void render_background(renderer_t *rend, const uint8_t col[4])
     vertices[2] = (vertex_t){{+1, +1, 0}, {c2[0], c2[1], c2[2], c2[3]}};
     vertices[3] = (vertex_t){{-1, +1, 0}, {c2[0], c2[1], c2[2], c2[3]}};
 
-    prog = get_prog(BACKGROUND_VSHADER, BACKGROUND_FSHADER, NULL);
+    prog = get_prog("asset://data/shaders/background.glsl", NULL);
     GL(glUseProgram(prog->prog));
 
     GL(glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, g_index_buffer));
@@ -1040,236 +1052,3 @@ int render_get_default_settings(int i, char **name, render_settings_t *out)
     }
     return 5;
 }
-
-
-// #### All the shaders code #######
-
-/*
- * I followed those name conventions.  All the vectors are expressed in eye
- * coordinates.
- *
- *         reflection         light source
- *
- *               r              s
- *                 ^         ^
- *                  \   n   /
- *  eye              \  ^  /
- *     v  <....       \ | /
- *              -----__\|/
- *                  ----+----
- *
- *
- */
-
-static const char *VSHADER =
-    "                                                                   \n"
-    "attribute vec3 a_pos;                                              \n"
-    "attribute vec3 a_normal;                                           \n"
-    "attribute vec4 a_color;                                            \n"
-    "attribute vec2 a_bshadow_uv;                                       \n"
-    "attribute vec2 a_bump_uv;   // bump tex base coordinates [0,255]   \n"
-    "attribute vec2 a_uv;        // uv coordinates [0,1]                \n"
-    "uniform   mat4 u_model;                                            \n"
-    "uniform   mat4 u_view;                                             \n"
-    "uniform   mat4 u_proj;                                             \n"
-    "uniform   mat4 u_shadow_mvp;                                       \n"
-    "uniform   float u_pos_scale;                                       \n"
-    "                                                                   \n"
-    "varying   vec3 v_pos;                                              \n"
-    "varying   vec4 v_color;                                            \n"
-    "varying   vec2 v_bshadow_uv;                                       \n"
-    "varying   vec2 v_uv;                                               \n"
-    "varying   vec2 v_bump_uv;                                          \n"
-    "varying   vec3 v_normal;                                           \n"
-    "varying   vec4 v_shadow_coord;  // Used for shadow mapping.        \n"
-    "                                                                   \n"
-    "void main()                                                        \n"
-    "{                                                                  \n"
-    "    v_normal = a_normal;                                           \n"
-    "    v_color = a_color;                                             \n"
-    "    v_bshadow_uv = (a_bshadow_uv + 0.5) /                          \n"
-    "                          (16.0 * VOXEL_TEXTURE_SIZE);             \n"
-    "    v_pos = a_pos * u_pos_scale;                                   \n"
-    "    v_uv = a_uv;                                                   \n"
-    "    v_bump_uv = a_bump_uv;                                         \n"
-    "    gl_Position = u_proj * u_view * u_model * vec4(v_pos, 1.0);    \n"
-    "    v_shadow_coord = (u_shadow_mvp * u_model * vec4(v_pos, 1.0));  \n"
-    "}                                                                  \n"
-;
-
-static const char *FSHADER =
-    "                                                                   \n"
-    "#ifdef GL_ES                                                       \n"
-    "precision highp float;                                             \n"
-    "#endif                                                             \n"
-    "                                                                   \n"
-    "uniform   mat4 u_model;                                            \n"
-    "uniform   mat4 u_view;                                             \n"
-    "uniform   mat4 u_proj;                                             \n"
-    "                                                                   \n"
-    "// Light parameters                                                \n"
-    "uniform   vec3 u_l_dir;                                            \n"
-    "uniform  float u_l_int;                                            \n"
-    "// Material parameters                                             \n"
-    "uniform  float u_m_amb; // Ambient light coef.                     \n"
-    "uniform  float u_m_dif; // Diffuse light coef.                     \n"
-    "uniform  float u_m_spe; // Specular light coef.                    \n"
-    "uniform  float u_m_shi; // Specular light shininess.               \n"
-    "uniform  float u_m_smo; // Smoothness.                             \n"
-    "                                                                   \n"
-    "uniform sampler2D u_bshadow_tex;                                   \n"
-    "uniform sampler2D u_bump_tex;                                      \n"
-    "uniform float     u_bshadow;                                       \n"
-    "uniform sampler2D u_shadow_tex;                                    \n"
-    "uniform float     u_shadow_k;                                      \n"
-    "                                                                   \n"
-    "varying lowp vec3 v_pos;                                           \n"
-    "varying lowp vec4 v_color;                                         \n"
-    "varying lowp vec2 v_uv;                                            \n"
-    "varying lowp vec2 v_bshadow_uv;                                    \n"
-    "varying lowp vec2 v_bump_uv;                                       \n"
-    "varying lowp vec3 v_normal;                                        \n"
-    "varying vec4 v_shadow_coord;                                       \n"
-    "                                                                   \n"
-    "vec2 uv, bump_uv;                                                  \n"
-    "vec3 n, s, r, v, bump;                                             \n"
-    "float s_dot_n;                                                     \n"
-    "float l_amb, l_dif, l_spe;                                         \n"
-    "float bshadow;                                                     \n"
-    "float visibility;                                                  \n"
-    "vec2 PS[4]; // Poisson offsets used for the shadow map.            \n"
-    "int i;                                                             \n"
-    "                                                                   \n"
-    "void main()                                                        \n"
-    "{                                                                  \n"
-    "    // clamp uv so to prevent overflow with multismapling.         \n"
-    "    uv = clamp(v_uv, 0.0, 1.0);                                    \n"
-    "    s = u_l_dir;                                                   \n"
-    "    n = normalize((u_view * u_model * vec4(v_normal, 0.0)).xyz);   \n"
-    "    bump_uv = (v_bump_uv + 0.5 + uv * 15.0) / 256.0;               \n"
-    "    bump = texture2D(u_bump_tex, bump_uv).xyz - 0.5;               \n"
-    "    bump = normalize((u_view * u_model * vec4(bump, 0.0)).xyz);    \n"
-    "    n = mix(bump, n, u_m_smo);                                     \n"
-    "    s_dot_n = dot(s, n);                                           \n"
-    "    l_dif = u_m_dif * max(0.0, s_dot_n);                           \n"
-    "    l_amb = u_m_amb;                                               \n"
-    "                                                                   \n"
-    "    // Specular light.                                             \n"
-    "    v = normalize(-(u_view * u_model * vec4(v_pos, 1.0)).xyz);     \n"
-    "    r = reflect(-s, n);                                            \n"
-    "    l_spe = u_m_spe * pow(max(dot(r, v), 0.0), u_m_shi);           \n"
-    "    l_spe = s_dot_n > 0.0 ? l_spe : 0.0;                           \n"
-    "                                                                   \n"
-    "    bshadow = texture2D(u_bshadow_tex, v_bshadow_uv).r;            \n"
-    "    bshadow = sqrt(bshadow);                                       \n"
-    "    bshadow = mix(1.0, bshadow, u_bshadow);                        \n"
-    "    gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);                       \n"
-    "    gl_FragColor.rgb += (l_dif + l_amb) * u_l_int * v_color.rgb;   \n"
-    "    gl_FragColor.rgb += l_spe * u_l_int * vec3(1.0);               \n"
-    "    gl_FragColor.rgb *= bshadow;                                   \n"
-    "                                                                   \n"
-    "    // Shadow map.                                                 \n"
-    "    #ifdef SHADOW                                                  \n"
-    "    visibility = 1.0;                                              \n"
-    "    vec4 shadow_coord = v_shadow_coord / v_shadow_coord.w;         \n"
-    "    float bias = 0.005 * tan(acos(clamp(s_dot_n, 0.0, 1.0)));      \n"
-    "    bias = clamp(bias, 0.0, 0.015);                                \n"
-    "    shadow_coord.z -= bias;                                        \n"
-    "    PS[0] = vec2(-0.94201624, -0.39906216) / 1024.0;               \n"
-    "    PS[1] = vec2(+0.94558609, -0.76890725) / 1024.0;               \n"
-    "    PS[2] = vec2(-0.09418410, -0.92938870) / 1024.0;               \n"
-    "    PS[3] = vec2(+0.34495938, +0.29387760) / 1024.0;               \n"
-    "    for (i = 0; i < 4; i++)                                        \n"
-    "        if (texture2D(u_shadow_tex, v_shadow_coord.xy +            \n"
-    "           PS[i]).z < shadow_coord.z) visibility -= 0.2;           \n"
-    "    if (s_dot_n <= 0.0) visibility = 0.5;                          \n"
-    "    gl_FragColor.rgb *= mix(1.0, visibility, u_shadow_k);          \n"
-    "    #endif                                                         \n"
-    "                                                                   \n"
-    "}                                                                  \n"
-;
-
-static const char *POS_DATA_VSHADER =
-    "                                                                   \n"
-    "attribute vec3 a_pos;                                              \n"
-    "attribute vec2 a_pos_data;                                         \n"
-    "uniform   mat4 u_model;                                            \n"
-    "uniform   mat4 u_view;                                             \n"
-    "uniform   mat4 u_proj;                                             \n"
-    "varying   vec2 v_pos_data;                                         \n"
-    "void main()                                                        \n"
-    "{                                                                  \n"
-    "    vec3 pos = a_pos;                                              \n"
-    "    gl_Position = u_proj * u_view * u_model * vec4(pos, 1.0);      \n"
-    "    v_pos_data = a_pos_data;                                       \n"
-    "}                                                                  \n"
-;
-
-static const char *POS_DATA_FSHADER =
-    "                                                                 \n"
-    "#ifdef GL_ES                                                     \n"
-    "precision highp float;                                           \n"
-    "#endif                                                           \n"
-    "                                                                 \n"
-    "varying lowp vec2 v_pos_data;                                    \n"
-    "uniform lowp vec2 u_block_id;                                    \n"
-    "                                                                 \n"
-    "void main()                                                      \n"
-    "{                                                                \n"
-    "    gl_FragColor.rg = u_block_id;                                \n"
-    "    gl_FragColor.ba = v_pos_data;                                \n"
-    "}                                                                \n"
-;
-
-static const char *SHADOW_MAP_VSHADER =
-    "                                                                   \n"
-    "attribute vec3 a_pos;                                              \n"
-    "uniform   mat4 u_model;                                            \n"
-    "uniform   mat4 u_view;                                             \n"
-    "uniform   mat4 u_proj;                                             \n"
-    "uniform   float u_pos_scale;                                       \n"
-    "void main()                                                        \n"
-    "{                                                                  \n"
-    "    gl_Position = u_proj * u_view * u_model *                      \n"
-    "                   vec4(a_pos * u_pos_scale, 1.0);                 \n"
-    "}                                                                  \n"
-;
-
-static const char *SHADOW_MAP_FSHADER =
-    "                                                                   \n"
-    "#ifdef GL_ES                                                       \n"
-    "precision highp float;                                             \n"
-    "#endif                                                             \n"
-    "                                                                   \n"
-    "void main()                                                        \n"
-    "{                                                                  \n"
-    "}                                                                  \n"
-;
-
-static const char *BACKGROUND_VSHADER =
-    "                                                                   \n"
-    "attribute vec3 a_pos;                                              \n"
-    "attribute vec4 a_color;                                            \n"
-    "                                                                   \n"
-    "varying vec4 v_color;                                              \n"
-    "                                                                   \n"
-    "void main()                                                        \n"
-    "{                                                                  \n"
-    "    gl_Position = vec4(a_pos, 1.0);                                \n"
-    "    v_color = a_color;                                             \n"
-    "}                                                                  \n"
-;
-
-static const char *BACKGROUND_FSHADER =
-    "                                                                   \n"
-    "#ifdef GL_ES                                                       \n"
-    "precision mediump float;                                           \n"
-    "#endif                                                             \n"
-    "                                                                   \n"
-    "varying vec4 v_color;                                              \n"
-    "                                                                   \n"
-    "void main()                                                        \n"
-    "{                                                                  \n"
-    "    gl_FragColor = v_color;                                        \n"
-    "}                                                                  \n"
-;
