@@ -28,7 +28,7 @@ static int iter(tool_t *tool, const float viewport[4])
     curs->snap_mask = SNAP_MESH;
     curs->snap_offset = 0;
 
-    goxel_set_help_text(goxel, "Click on the mesh to set plane.");
+    goxel_set_help_text("Click on the mesh to set plane.");
 
     if (curs->snaped && (curs->flags & CURSOR_PRESSED)) {
         curs->pos[0] = round(curs->pos[0]);
@@ -59,5 +59,5 @@ static int gui(tool_t *tool)
 TOOL_REGISTER(TOOL_SET_PLANE, plane, tool_plane_t,
               .iter_fn = iter,
               .gui_fn = gui,
-              .shortcut = "P"
+              .default_shortcut = "P"
 )
